package org.hansv.db;

import java.sql.*;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.h2.jdbc.*;
import org.h2.jdbcx.JdbcConnectionPool;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.FileDir;
import org.hansv.utils.Messages;

/**
 * Manage data from H2 database
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class H2db {
    private Connection conn;
    private Statement stmt;
    private FileDir filedir;
    private ErrorLog errlog;
    private Messages msg;
    

    public H2db(){ 
        
    }
      
     /**
     * Setter injector for class FileDir
     * @param fd Class FileDir
	 * @see org.hansv.utils.FileDir
     */
    public void setFileDir(FileDir fd){
        filedir = fd;
    }
    
    /**
     * Setter injector for class ErrorLog
     * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
     */
    public void setErrorLog(ErrorLog er){
        errlog = er;
    }
    
    /**
     * Setter injector for class Messages
     * @param m Class Messages
	 * @see org.hansv.utils.Messages
     */
    public void setMessages(Messages m){
        msg  = m;
    }
    
	/**
	 * Return database connection
	 * @return Connection
	 */
    public Connection getConn(){
        String path;
        try{
            path = filedir.getAppDirPath()+System.getProperty("file.separator") + "temp";
            Class.forName("org.h2.Driver");
            JdbcConnectionPool cp = JdbcConnectionPool.create("jdbc:h2:" + path , "sa", "");
            conn = cp.getConnection();
        }catch(JdbcSQLException sqlE){
            String descr = "Databáze H2 je pravděpodobně používána. \nPro odstranění problému zavřete jiné instance tohoto programu a nebo programy, které jsou na ní připojené, H2.H2.getConn";
            errlog.logErr(ExceptionUtils.getStackTrace(sqlE), descr);
            String[] errs = ExceptionUtils.getRootCauseStackTrace(sqlE);
            msg.showErr(errs[0] + errs[1], descr);
        }catch(SQLException|ClassNotFoundException e){
            String descr = "Chyba modulu pro import dat z mdb souboru do databáze H2, H2.getConn";
            errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
            String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
            msg.showErr(errs[0] + errs[1], descr);
        }
        return conn;
    }
	
	/**
	 * Close database connection
	 */
    public void closeConn(){
        try{
            conn.close();
        }catch(Exception e){
            String descr = "Chyba uzavření spojení s H2 databází, H2.closeConn";
            errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
            String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
            msg.showErr(errs[0] + errs[1], descr);
        }
    }
}


