package org.hansv.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.hansv.db.H2db;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class LocalityModel {

	private H2db db;
	
	/**
	 * Setter injector for class H2db
	 * @param h2 Class H2
	 * @see org.hansv.db.H2db
	 */
	public void setH2db(H2db h2){
		db = h2;
	}

	/**
	 * Drop table localities
	 * @throws SQLException
	 */
	public void dropTableLocalities() throws SQLException{
		Connection cnn = db.getConn();
		Statement stmt = cnn.createStatement();
		stmt.execute("DROP TABLE IF EXISTS localities;");
		cnn.close();
	}
	
	/**
	 * Create tables localities
	 * @throws SQLException
	 */
	public void createTableLocalities() throws SQLException{
		String locality = "CREATE TABLE localities("
				+ "id bigint auto_increment primary key," 
				+ "idloc VARCHAR(100),"
				+ "area VARCHAR(75),"
				+ "phytochorionorig VARCHAR(1000),"
				+ "kvadrant VARCHAR(7),"
				+ "locality VARCHAR(600),"
				+ "altitude SMALLINT,"
				+ "longitude double,"
				+ "latitude double,"
				+ "coordaccuracyorig VARCHAR(1000),"
				+ "coordsourceorig VARCHAR(1000),"
				+ "note VARCHAR(255),"
				+ "country VARCHAR(2),"
				+ "namedplaceorig VARCHAR(1000),"
				+ "importid INT)";
		Connection cnn = db.getConn();
		Statement stmt = cnn.createStatement();
		stmt.execute(locality);
		cnn.close();
	}
	
	/**
	 * Import data into table localities
	 * @throws SQLException
	 */
	public void importLocalities() throws SQLException{
		Connection cnn = db.getConn();
		String sql = "INSERT INTO localities (idloc, area, phytochorionorig, namedplaceorig, kvadrant, locality, latitude, longitude, coordaccuracyorig, coordsourceorig, altitude, note, importid, country) SELECT DISTINCT localityid ,area, phytochorion, namedplace, netmaparea, locality, latitude, longitude, coordaccuracy, coordsource, altitude, localitynote, importid, country FROM input";
		Statement stmt = cnn.createStatement();
		stmt.execute(sql);
		cnn.close();
	}
	
	/**
	 * Delete data in table localities
	 * @throws SQLException
	 */
	public void deleteLocalities() throws SQLException{
		Connection cnn = db.getConn();
		String sql = "DELETE FROM localities";
		Statement stmt = cnn.createStatement();
		stmt.execute(sql);
		cnn.close();
		
	}
	

	/**
	 * Set locality id in table localities
	 * @param localityid identifier of locality from server
	 * @param id identifier of record
	 * @throws SQLException
	 */
	public void setLocalityId(String localityid, int id) throws SQLException{
		Connection cnn = db.getConn();
		String sql = "UPDATE localities SET idloc = ? WHERE id = ?";
		PreparedStatement pstmt = cnn.prepareStatement(sql);
		pstmt.setString(1, localityid);
		pstmt.setInt(2, id);
		pstmt.execute();
		cnn.close();
	}
	
	
	
	public void addCollector(String guid, String collectorid) throws SQLException{
		String sql = "INSERT INTO plantauthors (plant,authors) VALUES (?,?)";
		Connection cnn = db.getConn();
		PreparedStatement pstmt = cnn.prepareStatement(sql);
		pstmt.setString(1, guid);
		pstmt.setString(2, collectorid);
		pstmt.execute();
		cnn.close();
	}
	
	public ResultSet getLocality() throws SQLException{
		Connection cnn = db.getConn();
		Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);
		ResultSet rs = stmt.executeQuery("SELECT * FROM localities WHERE idloc IS NULL");
		return rs;
	}
	
	/**
	 * Get count of localities
	 * @return 
	 */
	public int getCount() throws SQLException{
		Connection cnn = db.getConn();
		Statement stmt = cnn.createStatement();
		ResultSet rs = stmt.executeQuery("SELECT count(*) FROM localities");
		rs.first();
		return rs.getInt(1);
	}
	
	public void deleteImportId() {
		Connection cnn = db.getConn();
		String sql = "UPDATE localities SET importid = null";
		try{
		Statement stmt = cnn.createStatement();
		stmt.execute(sql);
		cnn.close();
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
}
