package org.hansv.model;


import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import org.apache.commons.lang.exception.ExceptionUtils;


/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class PhytochorionModel extends MapTableModel implements IMapTable{

	
	public PhytochorionModel(){
		table = "phytochorion";
	}
	
	
	/**
	 * Get all data from table project
	 * @return ResultSet
	 */
	@Override
	public ResultSet getData(){
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			ResultSet rs = stmt.executeQuery("SELECT id,orig AS `Fytochorion`, mapname AS `Mapovaný fytochorion`, mapids AS `Kód mapovaného fytochorionu` FROM phytochorion");
			return rs;
		}catch(SQLException e){
			String descr = "Nezdařilo se načtení dat z tabulky " + table;
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
	
	/**
	 * Import unique phytochorions into table phytochorion
	 */
	@Override
	public void importData(){
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement();
			String sql = "INSERT INTO " + table + " (orig) SELECT DISTINCT phytochorion from input WHERE phytochorion IS NOT NULL AND LENGTH(phytochorion) > 0";
			stmt.execute(sql);
			db.closeConn();
		}catch(SQLException e){
			String descr = "Nezdařil se import jedinečných taxonů do tabulky "+ table;
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	@Override	
	protected ArrayList getRest(String sample){
		return rm.getPhytochorion(sample);
	}
	
	@Override
	public void save(String[] data) {
		super.save(data);
	}
}
