package org.hansv.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Properties;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.utils.Crypto;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.FileDir;
import org.hansv.utils.Messages;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class ProxyModel {
	private FileDir filedir;
	private ErrorLog errlog;
    private Messages msg;
	private Crypto crypto;
	
	public ProxyModel(){
	}
	
	/**
	 * Setter injector for class FileDir
	 * @param fd Class FileDir
	 * @see org.hansv.utils.FileDir
	 */
	public void setFileDir(FileDir fd){
		filedir = fd;
	}
	
	/**
	 * Setter injector for class ErrorLog
	 * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
	 */
	public void setErrorLog(ErrorLog er){
		errlog = er;
	}
	
	/**
	 * Setter injector for class Messages
	 * @param m Class Messages
	 * @see org.hansv.utils.Messages
	 */
	public void setMessages(Messages m){
		msg  = m;
	}
	
	/**
	 * Setter injector for class Crypto
	 * @param c Class Crypto
	 * @see org.hansv.utils.Crypto
	 */
	public void setCrypto(Crypto c){
		crypto  = c;
	}
	
	public HashMap<String,String> getConfig(){
		HashMap<String,String> config;
		config = new HashMap<>();
		InputStream is;
		Properties props = new Properties();
		String path = filedir.getAppDirPath() + "/config.xml";
		if(fileExists(path)){
			try {
				File f = new File(path);
				is = new FileInputStream(f);
				props.loadFromXML(is);
				config.put("host", props.getProperty("Host"));
				config.put("port", props.getProperty("Port"));
				if(props.getProperty("User").length() > 0){
					config.put("user", props.getProperty("User"));
				}else{
					config.put("user",null);
				}
				if(props.getProperty("Password").length() > 0){
					config.put("password", crypto.decrypt(props.getProperty("Password")));
				}else{
					config.put("password",null);
				}
			} catch (FileNotFoundException e) {
				errlog.logErr(ExceptionUtils.getStackTrace(e), "ProxyModel.getConfig");
			}catch(IOException e){
				errlog.logErr(ExceptionUtils.getStackTrace(e), "ProxyModel.getConfig");
			}
		}
		return config;
	}
	
	public void setConfig(String host, String port, String name, String pswd){
		Properties prop = new Properties();
		prop.setProperty("Host", setEmpty(host));
		prop.setProperty("Port", port);
		if(name != null){
			prop.setProperty("User", name);
		}else{
			prop.setProperty("User", "");
		}
		if(pswd != null){
			if(pswd.length() > 0){
				prop.setProperty("Password", crypto.encrypt(pswd));
			}else{
				prop.setProperty("Password", "");
			}
		}else{
			prop.setProperty("Password", "");
		}
		String path = filedir.getAppDirPath() + "/config.xml";
		File f = new File(path);
		try{
			OutputStream out = new FileOutputStream(f);
			prop.storeToXML(out, "Configuration file of VratickaImport");
		}catch(IOException e){
			String descr = "ProxyModel.setConfig";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	private boolean fileExists(String path){
		File file = new File(path);
		return file.isFile();
	}
	
	private String setEmpty(String value){
		if(value == null){
			return "";
		}else{
			return value;
		}
	}
}
