package org.hansv.model;


import java.io.File;
import java.io.FileWriter;
import org.hansv.utils.ProxySingleton;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.LoginSingleton;
import org.hansv.utils.Utils;
import org.json.JSONArray;
import org.json.JSONObject;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class RestModel {

	private ErrorLog errlog;
	private LoginSingleton ls;
	private ProxySingleton ps;
	private Utils utils;

	public RestModel(){
		ls = LoginSingleton.getInstance();
		ps = ProxySingleton.getInstance();
	}



	/**
	 * Setter injector for class ErrorLog
	 * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
	 */
	public void setErrorLog(ErrorLog er){
		errlog = er;
	}
	
	/**
	 * Setter injector for class Utils
	 * @param utils Class Utils
	 * @see org.hansv.utils.Utils
	 */
	public void setUtils(Utils utils){
		this.utils = utils;
	}

	
	public String getTheme()  throws IOException{
		CloseableHttpClient httpClient = HttpClientBuilder.create().build();
		URIBuilder ub = new URIBuilder();
		ub.setPath(utils.getServerPath() +  "service/theme");
		String json = getJsonGet(ub);
		if(json == null){
			return null;
		}
		JSONObject obj = new JSONObject(json);
		String theme = obj.getString("theme");
		httpClient.close();
		return theme.trim();
	}

	public LoginItem login(String name, String passwd) throws IOException{
		LoginItem li = new LoginItem();
		URIBuilder ub = new URIBuilder();
		ub.setPath(utils.getServerPath() + "service/login").setParameter("user", name).setParameter("password", passwd);
		JSONObject obj = new JSONObject(getJsonGet(ub));
		li.setStatus(obj.getBoolean("status"));
		li.setMessage(obj.getString("message"));
		return li;
	}

		
	public ArrayList<String[]> getNamedPlaces(String sample){
		return getJSON(sample, "service/namedplace","sample",ls.getName(),ls.getPassword());
	}
	
	public ArrayList<String[]> getPhytochorion(String sample){
		return getJSON(sample, "service/phytochorion","sample");
	}
	
	public ArrayList<String[]> isCountryCode(String sample){
		return getJSON(sample, "service/country","sample");
	}

	public String getCountry(double latitude, double longitude){
		return getJSONGeoName(latitude,longitude);
	}
	
	public ArrayList<String[]> getNamedPlaces2(String sample){
		return getJSON(sample, "service/namedplace2","name",ls.getName(),ls.getPassword());
	}
	
	public ArrayList<String[]> getSources(String sample){
		return getJSON(sample, "service/source","sample",ls.getName(),ls.getPassword());
	}
	
	public ArrayList<String[]> getCollectors(String sample){
		return getJSON(sample, "service/collector","sample",ls.getName(),ls.getPassword());
	}
	
	public ArrayList<String[]> getProjects(String sample){
		return getJSON(sample, "service/project","sample",ls.getName(),ls.getPassword());
	}
	
	public ArrayList<String[]> getTaxons(String sample){
		return getJSON(sample, "service/taxon","sample");
	}
	
	public ArrayList<String[]> getTaxons2(String sample){
		return getJSON(sample, "service/taxon2","name");
	}
	
		
	private ArrayList<String[]> getJSON(String sample, String fragmentPath, String paramName){
		ArrayList<String[]> list = new ArrayList<>();
		URIBuilder ub = null;
		try {
			ub = new URIBuilder();
			ub.setPath(utils.getServerPath() + fragmentPath).setParameter(paramName, sample);
			list = getJSON2(ub);
		} catch (Exception e) {
			String descr = "Exception, RestModel.getJSON";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
		}
		return list;
	}
	
	private ArrayList<String[]> getJSON(String sample, String fragmentPath, String paramName, String user, String password){
		ArrayList<String[]> list = new ArrayList<>(); 
		try {
			URIBuilder ub = new URIBuilder();
			ub.setPath(utils.getServerPath() +  fragmentPath).setParameter(paramName, sample).setParameter("user", user).setParameter("password", password);
			list = getJSON2(ub);
		} catch (Exception e) {
			String descr = "Exception, RestModel.getJSON";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
		}
		return list;
	}
			
			
			
	private ArrayList<String[]> getJSON2(URIBuilder ub){
		ArrayList<String[]> list = new ArrayList<>(); 
		String json = getJsonGet(ub);
		if(json.length() > 21){
			JSONArray array = new JSONArray(json);
			for (int i = 0; i < array.length(); i++) {
				JSONObject object = array.getJSONObject(i);
				String[] content = new String[4];
				content[0] = object.getString("id");
				content[1] = object.getString("name");
				if(object.length()== 4){
					content[2] = Double.toString(object.getDouble("longitude"));
					content[3] = Double.toString(object.getDouble("latitude"));
				}
				list.add(content);		
			}
		}
		return list;
	}

	private String getJSONGeoName(double latitude, double longitude){
		String code = null;
		URIBuilder ub = null;
		try {
			ub = new URIBuilder();
			ub.setPath("http://api.geonames.org/countryCodeJSON").setParameter("username", "vladimirhans").setParameter("lat", Double.toString(latitude)).setParameter("lng", Double.toString(longitude));
			code = getJSONGeoName2(ub);
		} catch (Exception e) {
			String descr = "Exception, RestModel.getJSON";
			e.printStackTrace();
		}
		return code;
	}

	private String getJSONGeoName2(URIBuilder ub){
		ArrayList<String[]> list = new ArrayList<>(); 
		String json = getJsonGet(ub);
		String code = null;
		if(json.length() > 21){
			JSONObject obj = new JSONObject(json);
			if(obj.length() == 4){
				code = obj.getString("countryCode");
			}else{
				code = "00";
			}
		}
		return code;
	}
	
	
	
	public RestItem saveLocalityOccurences(LocalityOccurencesItem item) throws IOException{
		String json = getJsonString(item);
		RestItem ri = sendPost(json,"localityoccurences");
		return ri;
	}
	
	public RestItem getCountImported(int importid) throws IOException{
		String json = "{\"user\":\"" + ls.getName() + "\",\"password\":\"" + ls.getPassword() + "\",\"importid\":" + importid +  "}";
		return sendPost(json, "countimported");
	}
	

	public String getJsonGet(URIBuilder ub){
		String json = null;
		try{
			if(ps.getHost() == null){
				json = getJsonGet1(ub);
			}
			if(ps.getHost() != null && ps.getUser() ==  null){
				json = getJsonGet1(ub, ps.getHost(), ps.getPort());
			}
			if(ps.getHost() != null && ps.getUser() != null){
				json = getJsonGet1(ub, ps.getHost(), ps.getPort(),ps.getUser(),ps.getPassword());
			}
		}catch(IOException e){
			String descr = "Exception, RestModel.getResponseGet";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
		}catch(RuntimeException e){
			String descr = "RestModel.getJsonGet";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
		}
		return json;
	}
	
	/**
	 * Return CloseableHttpResponse using proxy with authenticate
	 * @param ub UriBuilder with parameters
	 * @param host	Host of proxy
	 * @param port	Port of proxy
	 * @param user	User of proxy
	 * @param password	Passwors of proxy
	 * @return String
	 * @throws IOException 
	 */
	private String getJsonGet1(URIBuilder ub, String host, int proxyport, String proxyuser, String proxypassword) throws IOException{
		CredentialsProvider credsProvider = new BasicCredentialsProvider();
		String json = null;
		credsProvider.setCredentials(
			new AuthScope(host, proxyport),
			new UsernamePasswordCredentials(proxyuser, proxypassword));
		CloseableHttpClient httpclient = HttpClients.custom()
			.setDefaultCredentialsProvider(credsProvider).build();

		try {
			URI uri = ub.build();
			HttpHost target = new HttpHost(uri.getPath(), uri.getPort());
			HttpHost proxy = new HttpHost(host, proxyport);

			RequestConfig config = RequestConfig.custom()
				.setProxy(proxy)
				.build();
			
			HttpGet httpget = new HttpGet(uri);
			httpget.addHeader("aacept","application/json");
			httpget.setConfig(config);

			CloseableHttpResponse response = httpclient.execute(target, httpget);
			if(response.getStatusLine().getStatusCode() == 204){
				return null;
			}
			if(response.getStatusLine().getStatusCode() == 404){
				String descr = "Nelze se připojit k serveru. Zkontrolujte připojení, zde nepotřebujete nastvit proxy server" + response.getStatusLine().getStatusCode() + " " + response.getStatusLine().getReasonPhrase();
				RuntimeException e = new RuntimeException(descr);
				throw e;
			}
				if(response.getStatusLine().getStatusCode() != 200){
				String descr = "Failed : Http error code : " + response.getStatusLine().getStatusCode();
				RuntimeException e = new RuntimeException(descr);
				throw e;
			}
			json = EntityUtils.toString(response.getEntity());
		}catch(URISyntaxException e){
			String descr = "RestModel.getJsonGet1";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
		}finally {
			httpclient.close();
		}
		return json;
	}
	
	/**
	 * Return CloseableHttpResponse using proxy without authenticate
	 * @param ub UriBuilder with parameters
	 * @param host	Host of proxy
	 * @param port	Port of proxy
	 * @return String
	 * @throws IOException 
	 */
	private String getJsonGet1(URIBuilder ub, String host, int proxyport) throws IOException{
		CloseableHttpClient httpclient = HttpClients.createDefault();
		String json = null;
		try {
			URI uri = ub.build();
			HttpHost target = new HttpHost(uri.getPath(), uri.getPort());
			HttpHost proxy = new HttpHost(host, proxyport);
			RequestConfig config = RequestConfig.custom()
				.setProxy(proxy)
				.build();
			HttpGet httpget = new HttpGet(uri);
			httpget.addHeader("aacept","application/json");
			httpget.setConfig(config);
			CloseableHttpResponse response = httpclient.execute(target, httpget);
			if(response.getStatusLine().getStatusCode() == 204){
				return null;
			}
			if(response.getStatusLine().getStatusCode() == 404){
				String descr = "Nelze se připojit k serveru. Zkontrolujte připojení, zde nepotřebujete nastvit proxy server" + response.getStatusLine().getStatusCode() + " " + response.getStatusLine().getReasonPhrase();
				RuntimeException e = new RuntimeException(descr);
				throw e;
			}
				if(response.getStatusLine().getStatusCode() != 200){
				String descr = "Failed : Http error code : " + response.getStatusLine().getStatusCode();
				RuntimeException e = new RuntimeException(descr);
				throw e;
			}
			json = EntityUtils.toString(response.getEntity());
		}catch(URISyntaxException e){
			String descr = "RestModel.getJsonGet1";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
		}finally {
			httpclient.close();
		}
		return json;
	}
	
	/**
	 * Return CloseableHttpResponse
	 * @param ub UriBuilder with parameters
	 * @return String
	 * @throws IOException 
	 */
	public String getJsonGet1(URIBuilder ub) throws IOException{
		CloseableHttpClient httpClient = HttpClients.createDefault();
		 String json = null;
		try{
			URI uri = ub.build();
			HttpGet getRequest = new HttpGet(uri);
			getRequest.addHeader("aacept","application/json");
			CloseableHttpResponse response = httpClient.execute(getRequest);
			if(response.getStatusLine().getStatusCode() == 204){
				return null;
			}
			if(response.getStatusLine().getStatusCode() == 404){
				String descr = "Nelze se připojit k serveru. Zkontrolujte připojení, popř. nastvení proxy server" + response.getStatusLine().getStatusCode() + " " + response.getStatusLine().getReasonPhrase();
				RuntimeException e = new RuntimeException(descr);
				throw e;
			}
			
			if(response.getStatusLine().getStatusCode() != 200){
				String descr = "Failed : Http error code : " + response.getStatusLine().getStatusCode();
				RuntimeException e = new RuntimeException(descr);
				throw e;
			}
			json = EntityUtils.toString(response.getEntity());
		}catch(URISyntaxException|IOException e){
			String descr = "RestModel.getJsonGet1";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
		}finally{
			httpClient.close();
		}
		return json;
	}
	
	private RestItem sendPost(String json, String restService) throws IOException{
		RestItem ri = null;
		if(ps.getHost() == null){
			ri = sendPost1(json, restService);
		}
		if(ps.getHost() != null && ps.getUser() == null){
			ri = sendPost1(json, restService, ps.getHost(), ps.getPort());
		}
		if(ps.getHost() != null && ps.getUser() != null){
			ri = sendPost1(json, restService, ps.getHost(), ps.getPort(), ps.getUser(), ps.getPassword());
		}

		return ri;
	}
	
	/**
	 * Send POST and return json
	 * @param json JSON string
	 * @param restService fragment url
	 * @param proxyhost proxy host
	 * @param proxyport proxy port
	 * @param proxyuser proxy user
	 * @param proxypassword  proxy password
	 * @return String
	 * @throws IOException 
	 */
	public RestItem sendPost1(String json, String restService, String proxyhost,int proxyport, String proxyuser, String proxypassword) throws IOException {
		CloseableHttpResponse response = null;
		CredentialsProvider credsProvider = new BasicCredentialsProvider();
		RestItem ri = null;
		credsProvider.setCredentials(
			new AuthScope(proxyhost,proxyport),
			new UsernamePasswordCredentials(proxyuser, proxypassword));
		CloseableHttpClient httpclient = HttpClients.custom().setDefaultCredentialsProvider(credsProvider).build();
		try {
			HttpHost target = new HttpHost(utils.getServerHost(), utils.getServerPort(), utils.getServerProtocol());
			HttpHost proxy = new HttpHost(proxyhost, proxyport);

			RequestConfig config = RequestConfig.custom()
				.setProxy(proxy)
				.setConnectTimeout(10000)
				.build();
			HttpPost httpget = new HttpPost(utils.getServerPath() + "/service/" + restService);
			
			httpget.setEntity(new StringEntity(json, ContentType.APPLICATION_JSON));
			httpget.setConfig(config);

			response = httpclient.execute(target, httpget);
			String jsonout = EntityUtils.toString(response.getEntity());
			ri = getRestItem(jsonout,response.getStatusLine().getStatusCode());
			
		} finally {
			httpclient.close();
		}
		return ri;
	}
	
	
	
	/**
	 * Send POST and return json
	 * @param json JSON string
	 * @param restService fragment url
	 * @param proxyhost proxy host
	 * @param proxyport proxy port
	 * @return String
	 * @throws IOException 
	 */
	public RestItem sendPost1(String json, String restService, String proxyhost,int proxyport) throws IOException {
		CloseableHttpResponse response = null;
		CloseableHttpClient httpclient = HttpClients.createDefault();
		RestItem ri = null;
		try {
			HttpHost target = new HttpHost(utils.getServerHost(), utils.getServerPort(),utils.getServerProtocol());
			HttpHost proxy = new HttpHost(proxyhost, proxyport);
			RequestConfig config = RequestConfig.custom()
				.setProxy(proxy)
				.setConnectTimeout(10000)
				.build();
			HttpPost httpget = new HttpPost(utils.getServerPath() + "/service/" + restService);
			httpget.setEntity(new StringEntity(json, ContentType.APPLICATION_JSON));
			httpget.setConfig(config);
			response = httpclient.execute(target, httpget);
			String jsonout = EntityUtils.toString(response.getEntity());
			ri = getRestItem(jsonout,response.getStatusLine().getStatusCode());
		} finally {
			httpclient.close();
		}
		return ri;
	}
	
	
	
	/**
	 * Send POST and return json
	 * @param json JSON string
	 * @param restService fragment urlt
	 * @return String
	 * @throws IOException 
	 */
	private RestItem sendPost1(String json, String restService) throws IOException{
		RestItem ri = null;
		CloseableHttpResponse response = null;
		CloseableHttpClient httpclient = HttpClients.createDefault();

		HttpHost target = new HttpHost(utils.getServerHost(), utils.getServerPort(),utils.getServerProtocol());
		HttpPost httpget = new HttpPost("/" + utils.getServerPartPath() + "service/" + restService);
		RequestConfig config = RequestConfig.custom()
				.setConnectTimeout(10000)
				.build();
		httpget.setEntity(new StringEntity(json, ContentType.APPLICATION_JSON));
		httpget.setConfig(config);
		response = httpclient.execute(target, httpget);
		String jsonout = EntityUtils.toString(response.getEntity());
		ri = getRestItem(jsonout,response.getStatusLine().getStatusCode());
		return ri;
	}
	
	private static RestItem getRestItem(String json, int httpcode) throws IOException{
        HashMap<String, String> map = new HashMap<String, String>();
		writeJson(json);
		JSONObject jObject = new JSONObject(json);
		Iterator<?> keys = jObject.keys();
		while(keys.hasNext()){
			String key = (String)keys.next();
			String value = jObject.getString(key);
			map.put(key, value);
		}
		RestItem ri = new RestItem(map, httpcode);
		return ri;
	}
	
	/**
	 * Get json string from LocalityOccurenceItem
	 * @param loi
	 * @return String
	 */
	private String getJsonString(LocalityOccurencesItem loi){
		String json = "{";
		json += "\"locality\": \"" + StringEscapeUtils.escapeJava(loi.locality) + "\",";
		json += "\"coordsource\":\"" + loi.coordsource + "\",";
		json += "\"longitude\":\"" + loi.longitude + "\",";
		json += "\"latitude\":\"" + loi.latitude + "\",";
		json += "\"coordaccuracy\":\"" + loi.coordaccuracy + "\",";
		json += "\"importid\":\"" + loi.importid + "\",";
		json += "\"user\":\"" + loi.user + "\",";
		json += "\"password\":\"" + loi.password + "\"";
		if(loi.namedplace != null){
			if(loi.namedplace.equals("") == false){
				json += ",\"namedplace\":\"" + loi.namedplace + "\"";
			}
		}else{
			json += ",\"country\":\"" + loi.country + "\"";
		}
		
		if(loi.phytochorion != null){
			if(loi.phytochorion.equals("") ==false){
				json += ",\"phytochorion\":\"" + loi.phytochorion + "\"";
			}
		}
		if(loi.area != null){
			if(loi.area.equals("") == false){
				json += ",\"area\":\"" + StringEscapeUtils.escapeJava(loi.area) + "\"";
			}
		}
		if(loi.altitude != null){
			if(loi.altitude.equals("0") == false){
				json += ",\"altitude\":\"" + loi.altitude + "\"";
			}
		}
		if(loi.netmaparea != null){
			if(loi.netmaparea.equals("") == false){
				json += ",\"netmaparea\":\"" + StringEscapeUtils.escapeJava(loi.netmaparea) + "\"";
			}
		}
		if(loi.note != null){
			if(loi.note.equals("") == false){
				json += ",\"note\":\"" + StringEscapeUtils.escapeJava(loi.note) + "\"";
			}
		}
		json += ",\"occurences\":[";
		String jsonocc = "";
		for (OccurenceItem occurencesitem : loi.occurencesitem) {
			if(jsonocc.length() > 0){
				jsonocc += ",";
			}
			jsonocc += "{";
			jsonocc += "\"taxon\":\"" + occurencesitem.taxon + "\",";
			jsonocc += "\"date1\":\"" + occurencesitem.date1 + "\",";
			jsonocc += "\"source\":\"" + occurencesitem.source + "\",";
			jsonocc += "\"project\":\"" + occurencesitem.project + "\",";
			jsonocc += "\"collectors\":\"" + occurencesitem.collectors + "\"";
			if(occurencesitem.note != null){
				jsonocc += ",\"note\":\"" + StringEscapeUtils.escapeJava(occurencesitem.note) + "\"";
			}
			if(occurencesitem.schedanote != null){
				jsonocc += ",\"schedanote\":\"" + StringEscapeUtils.escapeJava(occurencesitem.schedanote) + "\"";
			}
			if(occurencesitem.schedacount != null){
				jsonocc += ",\"schedacount\":\"" + StringEscapeUtils.escapeJava(occurencesitem.schedacount) + "\"";
			}
			if(occurencesitem.herbarium != null){
				if(occurencesitem.herbarium.equals("") == false){
					jsonocc += ",\"herbarium\":\"" + StringEscapeUtils.escapeJava(occurencesitem.herbarium) + "\"";
				}
			}
			if(occurencesitem.originalname != null){
				if(occurencesitem.originalname.equals("") == false){
					jsonocc += ",\"originalname\":\"" + StringEscapeUtils.escapeJava(occurencesitem.originalname) + "\"";
				}
			}
			jsonocc += ",\"originalmappedname\":\"" + StringEscapeUtils.escapeJava(occurencesitem.originalmappedname) + "\"";
			if(occurencesitem.abbrev != null){
				if(occurencesitem.abbrev.equals("") == false){
					jsonocc += ",\"determinationqualificator\":\"" + StringEscapeUtils.escapeJava(occurencesitem.abbrev) + "\"";
				}
			}
			if(occurencesitem.abund != null){
				if(occurencesitem.abund.equals("") == false){
					jsonocc += ",\"abund\":\"" + StringEscapeUtils.unescapeJava(occurencesitem.abund) + "\"";
				}
			}
			if(occurencesitem.authorrev != null){
				if(occurencesitem.authorrev.equals("") == false){
					jsonocc += ",\"authorrev\":\"" + StringEscapeUtils.escapeJava(occurencesitem.authorrev) + "\"";
				}
			}
			if(occurencesitem.authordet != null){
				if(occurencesitem.authordet.equals("") == false){
					jsonocc += ",\"authordet\":\"" + StringEscapeUtils.escapeJava(occurencesitem.authordet) + "\"";
				}
			}
			if(occurencesitem.negative != null){
				if(occurencesitem.negative.equals("") == false){
					jsonocc += ",\"negative\":\"" + occurencesitem.negative + "\"";
				}
			}
			
			jsonocc += "}";
		}
		json += jsonocc;
		
		json += "]}";
		System.out.println(json);
		writeJson(json);
		return json;
	}
	
	private static void writeJson(String json){
		File f;
		String strLogFile;
		strLogFile = System.getProperty("user.home") + File.separator + ".vratickaimport/vratickaimport-json.log";
		try{
			f=new File(strLogFile);
			f.createNewFile();
			try (PrintWriter vystup = new PrintWriter(new FileWriter(strLogFile,true))) {
				vystup.println( json);
			}
		}catch(Exception e){
			e.printStackTrace();
		}
	}
}
