package org.hansv.utils;

import gnu.crypto.cipher.CipherFactory;
import gnu.crypto.cipher.IBlockCipher;
import gnu.crypto.util.Base64;
import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.lang.exception.ExceptionUtils;


/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class Crypto {
	private ErrorLog errlog;
	private Messages msg;
	private byte[] key;
	byte[] key_bytes;
	
	public Crypto(){
		key_bytes = "u9t1*6kl?zs#QNsP".getBytes();
	}
	
	/**
	 * Setter injector for class Messages
	* @param m Class Messages
	* @see org.hansv.utils.Messages
	*/
	public void setMessages(Messages m){
		msg  = m;
	}
	
	/**
	 * Setter injector for class ErrorLog
	 * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
	 */
	public void setErrorLog(ErrorLog er){
		errlog = er;
	}
	
	/**
	 * Encrypt string
	 * @param input Encryptet string
	 * @return String
	 */
	public String encrypt(String input) {
		try {
			IBlockCipher cipher = CipherFactory.getInstance("AES");
			Map attributes = new HashMap();
			attributes.put(IBlockCipher.CIPHER_BLOCK_SIZE, new Integer(16));
			attributes.put(IBlockCipher.KEY_MATERIAL, key_bytes);
			cipher.init(attributes);
			int bs = cipher.currentBlockSize();
			for(int i = 0; 0 != input.getBytes().length % bs ;i++){
				input += "\t";
			}
			input += "\t";
			byte[] pt =  input.getBytes();
			byte[] ct = new byte[pt.length] ;
			for (int i = 0; i + bs < pt.length; i += bs)
			{
				cipher.encryptBlock(pt, i, ct, i);
			}
			return Base64.encode(ct);
		} catch (InvalidKeyException|IllegalStateException e) {
			String descr = "Nezdařilo se dekódovat řetězec";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return null;
	}
	
	/**
	 * Decryptet string
	 * @param input Decrypted string
	 * @return String
	 */
	public String decrypt(String input) {
		try {
			IBlockCipher cipher = CipherFactory.getInstance("AES");
			Map attributes = new HashMap();
			attributes.put(IBlockCipher.CIPHER_BLOCK_SIZE, new Integer(16));
			attributes.put(IBlockCipher.KEY_MATERIAL, key_bytes);
			cipher.init(attributes);
			int bs = cipher.currentBlockSize();
			byte[] input2 = null;
			try {
				input2 = Base64.decode(input);
			} catch (UnsupportedEncodingException ex) {
				Logger.getLogger(Crypto.class.getName()).log(Level.SEVERE, null, ex);
			}
			byte[] cpt =  new byte[input2.length];
			for (int i = 0; i + bs < cpt.length; i += bs)
			{
				cipher.decryptBlock(input2, i, cpt, i);
			}
			
			return new String(cpt).trim();
		} catch (InvalidKeyException|IllegalStateException e) {
			String descr = "Nezdařilo se zakódovat řetězec";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return null;
	}
}
