package org.hansv.utils;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.db.H2db;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class DbUtils {
    private H2db db;
    private ErrorLog errlog;
    private Messages msg;
    
    public DbUtils(){
        
    }
    
    /**
     * Setter injector for class ErrorLog
     * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
     */
    public void setErrorLog(ErrorLog er){
        errlog = er;
    }
    
    /**
     * Setter injector for class Messages
     * @param m Class Messages
	 * @see org.hansv.utils.Messages
     */
    public void setMessages(Messages m){
        msg  = m;
    }
    
    /**
     * Setter injector for class H2db
     * @param h2 Class H2db
	 * @see org.hansv.db.H2db
     */
    public void setH2db(H2db h2){
        db  = h2;
    }
    
    /**
     * Check if exists table
     * @param table Name of database table
     * @return boolean
     */
    public boolean tableExist(String table){
        boolean result = false;
        Connection cnn = db.getConn();
        try {
            DatabaseMetaData dbmtdt = cnn.getMetaData();
            ResultSet rs = dbmtdt.getTables(null, null, table, null);
            if(rs.next()){
                result = true;
            }
        } catch (SQLException e) {
            String descr = "Nezdařilo se získat informace o tabulce " + table;
            errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
            String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
            msg.showErr(errs[0] + errs[1], descr);
        }
        db.closeConn();
        return result;
    }
	
	/**
	 * Get type of specific column of table
	 * @param table Name of database table
	 * @param column Name of column in database table
	 * @return String
	 */
	public String getColumnType(String table, String column){
		String result = null;
		if(tableExist(table) == false){
			String descr = "Tabulka " + table + " nebyla nalezena";
            msg.showErr(null, descr);
			return result;
		}
		Connection cnn = db.getConn();
		try{
			DatabaseMetaData dbmtdt = cnn.getMetaData();
			ResultSet rs  = dbmtdt.getColumns(null, null, table, column);
			rs.beforeFirst();
			while(rs.next()){
				result = rs.getString(6);
			}
		}catch (SQLException e) {
			e.printStackTrace();
            String descr = "Nezdařilo se získat informace o tabulce " + table;
            errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
            String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
            msg.showErr(errs[0] + errs[1], descr);
        }
        db.closeConn();
		return result;
	}
}
