package org.hansv.utils;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.jnlp.BasicService;
import javax.jnlp.ServiceManager;
import javax.jnlp.UnavailableServiceException;
import org.apache.commons.lang.exception.ExceptionUtils;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 */
public class Utils {
	
	private ErrorLog errlog;
	private Messages msg;
	
	public Utils(){
	}
	
	public void setUp(){
	
	}
	
	/**
	 * Setter injector for class Messages
	 * @param m Class Messages
	 * @see org.hansv.utils.Messages
	 */
	public void setMessages(Messages m){
		msg  = m;
	}
	
	/**
	 * Setter injector for class ErrorLog
	 * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
	 */
	public void setErrorLog(ErrorLog er){
		errlog = er;
	}
	
	/**
	 * Return path of server, where is stored applet and running via java web start
	 * @return String
	 */
	public String getServerPath(){
		String path = "";
		try{
			BasicService bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");
			final URL codeBase = bs.getCodeBase();
			String pathApplet = codeBase.toString();
			int end = pathApplet.indexOf("/applets/");
			path = pathApplet.substring(0,end+1);
		}catch(UnavailableServiceException e){
			path = "http://192.168.122.173/vraticka/www/";
		}catch(Exception e){
			String descr = "Ziskani cesty k serveru, Utils.utils";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return path;
	}
	
	
	/**
	 * Return port of server
	 * @return int
	 */
	public int getServerPort(){
		int port = 0;
		try{
			BasicService bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");
			final URL codeBase = bs.getCodeBase();
			port = codeBase.getDefaultPort();
		}catch(UnavailableServiceException e){
			port = 80;
		}catch(Exception e){
			String descr = "Ziskani portu serveru, Utils.utils";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return port;
	}
	
	/**
	 * Return host
	 * @return String
	 */
	public String getServerHost(){
		String host = null;
		try{
			BasicService bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");
			final URL codeBase = bs.getCodeBase();
			host = codeBase.getHost();
		}catch(UnavailableServiceException e){
			host = "192.168.122.173";
		}catch(Exception e){
			String descr = "Ziskani nazvu serveru, Utils.getServerHost";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return host;
	}
	
	/**
	 * Return path without host name
	 * @return String
	 */
	public String getServerPartPath(){
		String path = null;
		try{
			BasicService bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");
			final URL codeBase = bs.getCodeBase();
			path = codeBase.getPath();
			int end = path.indexOf("/applets/");
			path = path.substring(0,end+1);
		}catch(UnavailableServiceException e){
			path = "vraticka/www/";
		}catch(Exception e){
			String descr = "Ziskani cesty, Utils.getServerPath3";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return path;
	}
	
	/**
	 * Return of protocol of server
	 * @return String
	 */
	public String getServerProtocol(){
		String protocol = null;
		try{
			BasicService bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");
			final URL codeBase = bs.getCodeBase();
			protocol = codeBase.getProtocol();
		}catch(UnavailableServiceException e){
			protocol = "http";
		}catch(Exception e){
			String descr = "Ziskani portu serveru, Utils.utils";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return protocol;
	}
	
	/**
	 * Return duplicit value from ArrayList
	 * @param al List
	 * @return List
	 */
	public List getDuplicitInList(List al){
		int i;
		List<String> duplicit = new ArrayList<>(); 
		for (Object o:al){
			i = 0;
			for(Object ob:al){
				if(ob.toString().equals(o.toString())){
					i++;
				}
			}
			if(i>1){
				if(duplicit.contains(o.toString()) == false){
					duplicit.add(o.toString());
				}
			}
		}
		
		return duplicit;
	}
	
	/**
	 * Check if string is integer
	 * @param s Checked string
	 * @return boolean
	 */
	public boolean isInteger(String s) {
		boolean result = false;
		Pattern p = Pattern.compile("^-?[0-9]+$");
		Matcher m = p.matcher(s);
		if(m.matches()){
			result = true;
		}
		return result;
	}
	
	/**
	 * Check if string is double
	 * @param s Checked string
	 * @return boolean
	 */
	public boolean isDouble(String s){
		Pattern DOUBLE_PATTERN = Pattern.compile(
			"[\\x00-\\x20]*[+-]?(NaN|Infinity|((((\\p{Digit}+)(\\.)?((\\p{Digit}+)?)" +
			"([eE][+-]?(\\p{Digit}+))?)|(\\.((\\p{Digit}+))([eE][+-]?(\\p{Digit}+))?)|" +
		"(((0[xX](\\p{XDigit}+)(\\.)?)|(0[xX](\\p{XDigit}+)?(\\.)(\\p{XDigit}+)))" +
		"[pP][+-]?(\\p{Digit}+)))[fFdD]?))[\\x00-\\x20]*");
		return DOUBLE_PATTERN.matcher(s).matches();
	}

	public boolean isNegative(String value){
		boolean res = false;
		if(isInteger(value) == true){
			if(Integer.parseInt(value) == 0 || Integer.parseInt(value) == 1){
				res = true;
			}
		}else{
			if(value.toLowerCase().equals("true") || value.toLowerCase().equals("false")){
				res = true;
			}
		}
		return res;
	}
	
	/**
	 * Transform format of latitude
	 * @param coord Coordinate
	 * @return Double
	 */
	public Double transformLatitude(String coord){
		Double lat = transformCoord(coord);
		if(lat == null){
			return null;
		}
		if(lat < -90 || lat > 90){
			return null;
		}
		return lat;
	}
	
	/**
	 * Transform format of longitude
	 * @param coord Coordinate
	 * @return Double
	 */
	public Double transformLongitude(String coord){
		Double longitude = transformCoord(coord);
		if(longitude == null){
			return null;
		}
		if(longitude < -180 || longitude > 180){
			return null;
		}
		return longitude;
	}
	
	/**
	 * Transform format of coordinates
	 * @param coord Coordinate
	 * @return boolean
	 */
	private Double transformCoord(String coord){
		Double result = null;
		coord = coord.replace(" ", "");
		Pattern p = Pattern.compile("^[0-1]?[0-9]{1,2}:[0-5]?[0-9]:[0-5]?[0-9][.,][0-9]*[N,S,E,W]?$");
		Matcher m = p.matcher(coord);
		if(m.matches()){
			result = transformCoord1(coord);
		}
		Pattern p1 = Pattern.compile("^[-]?[0-3]?[0-9]{1,2}[.,][0-9]*[N,S,E,W]?$");
		Matcher m1 = p1.matcher(coord);
		if(m1.matches()){
			result = transformCoord2(coord);
		}
		Pattern p2 = Pattern.compile("^[0-1]?[0-9]{1,2}°[0-5]?[0-9][',′,’][0-5]?[0-9][.,][0-9]*[\",'',″][N,S,E,W]?$");
		Matcher m2 = p2.matcher(coord);
		if(m2.matches()){
			result = transformCoord3(coord);
		}
		Pattern p3 = Pattern.compile("^[N,S,E,W][0-3]?[0-9]{1,2}°[0-5]?[0-9]{1,2}[.,][0-9]*[',′,’]$");
		Matcher m3 = p3.matcher(coord);
		if(m3.matches()){
			result = transformCoord4(coord);
		}
		Pattern p4 = Pattern.compile("^[0-1]?[0-9]{1,2}:[0-5]?[0-9]:[0-5]?[0-9][N,S,E,W]$");
		Matcher m4 = p4.matcher(coord);
		if(m4.matches()){
			result = transformCoord1(coord);
		}
		Pattern p5 = Pattern.compile("^[0-1]?[0-9]{1,2}:[0-5]?[0-9]:[0-5]?[0-9]$");
		Matcher m5 = p5.matcher(coord);
		if(m5.matches()){
			result = transformCoord1(coord);
		}
		Pattern p6 = Pattern.compile("^[0-1]?[0-9]{1,2}:[0-5][0-9]:[0-5][0-9][.,][0-9]*$");
		Matcher m6 = p6.matcher(coord);
		if(m6.matches()){
			result = transformCoord1(coord);
		}
		Pattern p7 = Pattern.compile("^[0-1]?[0-9]{1,2}°[0-5]?[0-9][',′,’][0-5]?[0-9][\",'',″][N,S,E,W]?$");
		Matcher m7 = p7.matcher(coord);
		if(m7.matches()){
			result = transformCoord3(coord);
		}
		return result;
	}
	
	/**
	 * Transfor coordinates from form dd:mm:ss[N,S,W,E] to double
	 * @param coord Coordinate
	 * @return double
	 */
	private double transformCoord1(String coord){
		Double multiplier = 1.0;
		if(coord.endsWith("S") || coord.endsWith("W")){
			multiplier = -1.0;
			coord = coord.substring(0, coord.length()-1);
		}
		if(coord.endsWith("N") || coord.endsWith("E")){
			coord = coord.substring(0, coord.length()-1);
		}
		coord = coord.replace(",", ".");
		String[] partsCoord = coord.split(":");
		Double deg = Double.parseDouble(partsCoord[0]);
		Double min = Double.parseDouble(partsCoord[1])/60;
		Double sec = Double.parseDouble(partsCoord[2])/3600;
		Double dblCoord = (deg + min + sec) * multiplier;
		return dblCoord;
	}
	
	/**
	 * Transfor coordinates from form dd.ddd[N,S,W,E] to double
	 * @param coord Coordinate
	 * @return Double
	 */
	private Double transformCoord2(String coord){
		Double multiplier = 1.0;
		if(coord.startsWith("-")){
			multiplier *= -1;
			coord = coord.substring(1, coord.length());
		}
		if(coord.endsWith("S") || coord.endsWith("W")){
			multiplier *= -1.0;
			coord = coord.substring(0, coord.length()-1);
		}
		if(coord.endsWith("N") || coord.endsWith("E")){
			coord = coord.substring(0, coord.length()-1);
		}
		coord = coord.replace(",", ".");
		Double dblCoord = Double.parseDouble(coord) * multiplier;
		return dblCoord;
	}
	
	/**
	 * Transfor coordinates from form dd°mm'ss''[N,S,W,E] to double
	 * @param coord Coordinate
	 * @return Double
	 */
	private Double transformCoord3(String coord){
		Double multiplier = 1.0;
		coord = coord.replace(",", ".");
		coord = coord.replace("′", "'");
		coord = coord.replace("’", "'");
		coord = coord.replace("\"", "");
		coord = coord.replace("″", "");
		coord = coord.replace("''", "");
		
		if(coord.endsWith("S") || coord.endsWith("W")){
			multiplier = -1.0;
			coord = coord.substring(0, coord.length()-1);
		}
		if(coord.endsWith("N") || coord.endsWith("E")){
			coord = coord.substring(0, coord.length()-1);
		}
		
		Double deg = Double.parseDouble(coord.substring(0,coord.indexOf("°")));
		Double min = Double.parseDouble(coord.substring(coord.indexOf("°")+1, coord.indexOf("'")))/60;
		Double sec = Double.parseDouble(coord.substring(coord.indexOf("'") +1, coord.length()))/3600;
		Double dblCoord = (deg + min + sec) * multiplier;
		return dblCoord;
	}
	
	/**
	 * Transfor coordinates from form [N,S,W,E]dd°mm' to double
	 * @param coord Coordinate
	 * @return Double
	 */
	private Double transformCoord4(String coord){
		Double multiplier = 1.0;
		coord = coord.replace(",", ".");
		coord = coord.replace("′", "'");
		if(coord.startsWith("S") || coord.startsWith("W")){
			multiplier = -1.0;
			coord = coord.substring(1, coord.length());
		}
		if(coord.startsWith("N") || coord.startsWith("E")){
			coord = coord.substring(0, coord.length()-1);
		}
		
		Double deg = Double.parseDouble(coord.substring(0,coord.indexOf("°")));
		Double min = Double.parseDouble(coord.substring(coord.indexOf("°")+1, coord.indexOf("'")))/60;
		Double dblCoord = (deg + min) * multiplier;
		return dblCoord;
	}
	
	
	/**
	 * Transform format of date
	 * @param date Date
	 * @return String
	 */
	public String transformDate(String date){
		String result = "";
		if(date != null){
			Pattern p = Pattern.compile("^([1][8,9][0-9]{2}|[2][0,1,2][0-9]{2})-([0]?[1-9]|1[0-2])-([0]?[1-9]|[1,2][0-9]|[3][0,1])$");
			Matcher m = p.matcher(date);
			if(m.matches()){
				result = date;
			}
			Pattern p1 = Pattern.compile("^([1][8,9][0-9]{2}|[2][0,1,2][0-9]{2})$");
			Matcher m1 = p1.matcher(date);
			if(m1.matches()){
				result = date + "-00-00";
			}
			Pattern p2 = Pattern.compile("^([1][8,9][0-9]{2}|[2][0,1,2][0-9]{2})-([0]?[1-9]|1[0-2])$");
			Matcher m2 = p2.matcher(date);
			if(m2.matches()){
				result = date + "-00";
			}
			Pattern p3 = Pattern.compile("^([0-3]?[0-9])\\.\\s?([0]?[1-9]|1[0-2])\\.\\s?([1][8,9][0-9]{2}|[2][0,1,2][0-9]{2})$");
			Matcher m3 = p3.matcher(date);
			if(m3.matches()){
				result = transformDate1(date);
			}
			Pattern p4 = Pattern.compile("^(I{1,3}|IV|V[I]{0,3}|IX|X[I]{0,2})\\.\\s?([1][8,9][0-9]{2}|[2][0,1,2][0-9]{2})$");
			Matcher m4 = p4.matcher(date);
			if(m4.matches()){
				result = transformDate2(date);
			}
			Pattern p5 = Pattern.compile("^s.d.$");
			Matcher m5 = p5.matcher(date);
			if(m5.matches()){
				result = "1000-01-01";
			}
		}
		return result;
	}
	
	private String transformDate1(String date){
		date = date.replace(" ", "");
		date = date.replace(".", "#");
		String[] dateParts = date.split("#");
		return dateParts[2]+"-" + getCorrectPartDate(dateParts[1]) + "-" + getCorrectPartDate(dateParts[0]);
	}

	private String transformDate2(String date){
		String[] date2 = date.split("\\.");
		String month = "";
		switch(date2[0].trim()){
			case "I":
				month = "01";
				break;
			case "II":
				month = "02";
				break;
			case "III":
				month = "03";
				break;
			case "IV":
				month = "04";
				break;
			case "V":
				month = "05";
				break;
			case "VI":
				month = "06";
				break;
			case "VII":
				month = "07";
				break;
			case "VIII":
				month = "08";
				break;
			case "IX":
				month = "09";
				break;
			case "X":
				month = "10";
				break;
			case "XI":
				month = "11";
				break;
			case "XII":
				month = "12";
				break;
		}
		return date2[1].trim() + "-" + month + "-00";
	}
	
	private String getCorrectPartDate(String part){
		String res;
		if(part.length() == 1){
			res = "0" + part;
		}else{
			res = part;
		}
		return res;
	}
		
	
	
	
	/**
	 * Return first value from ArrayList with String[2] 
	 * @param name Searched string
	 * @param list ArrayList
	 * @return String
	 */
	public String getKey(String name, ArrayList list){
		String key = null;
		for(int i = 0; i < list.size();i++){
			String[] values = (String[])list.get(i);
			if(name.equals(values[1])){
				key = values[0];
			}
		}
		return key;
	}
	
	/**
	 * Get HTML header
	 * @return String
	 */
	public String getHtmlHeader(){
		SettingSingleton setSin = SettingSingleton.getInstance();
		String html =	"<!DOCTYPE html>\n" +
						"<html>\n" +
						"	<head>\n" +
						"		<meta charset=\"utf-8\">" +
						"		<link rel='stylesheet' media='screen,projection,tv' href='" + setSin.getUrlTheme() + "'>" +
						"	</head><body><div id='content'>";
		return html;
	}
	
	/**
	 * Get HTML foot
	 * @return String
	 */
	public String getHtmlFoot(){
		return "</div></body></html>";
	}
	
	/**
	 * Decode returned http code
	 * @param code
	 * @return String
	 */
	public String decodeHttpCode(int code){
		switch(code){
			case(400):
				return "Chybný požadavek. Zkuste spusti znovu import";
			case(401):
				return "Chybí přihlašovací jméno nebo heslo";
			case(402):
				return "Chybí povinné položky";
			case(403):
				return "Chybné přihlašovací jméno nebo heslo";
			case(404):
				return "Adresa nenalezena. Zkontrolujte připojení k síti, popř. nastavení proxy serveru";
			case(405):
				return "Hodnota nějakého parametru není přijatelná. Zkontrolujte, zda všechny mapované hodnoty odpovídají hodnotám na serveru";
			case(407):
				return "Proxy server vyžaduje autentizaci";
			case(406):
				return "Nějaká hodnota nebyla akceptována";
			case(408):
				return "Vypršel čas pro požadavek";
			case(409):
				return "Nezdařilo se vytvoření lokalit/nálezu/nálezce";
			case(502):
				return "Chyba brány a nebo proxy serveru. Zkuste import znovu. Pokud se opět vyskytne tato chyba, kontaktujte administrátora sítě";
			case(503):
				return "Nedostupná služba. Zkuste import znovu. Pokud se opět vyskytne tato chyba, kontaktujte administrátora sítě";
			default:
				return null;
		}
	}
	
	/**
	 * Get distance between 2 points bz haversine algorithm
	 * @param lat1	Latitude of first point
	 * @param lon1	Longitude of first point
	 * @param lat2	Latitude of second point
	 * @param lon2	Longitude of second point
	 * @return double
	 */
	public double getDistance(double lat1, double lon1, double lat2, double lon2) {
		double R = 6372.8;
		double dLat = Math.toRadians(lat2 - lat1);
		double dLon = Math.toRadians(lon2 - lon1);
		lat1 = Math.toRadians(lat1);
		lat2 = Math.toRadians(lat2);
		double a = Math.pow(Math.sin(dLat / 2),2) + Math.pow(Math.sin(dLon / 2),2) * Math.cos(lat1) * Math.cos(lat2);
		double c = 2 * Math.asin(Math.sqrt(a));
		double distance = Math.round(R * c*1000);
		return distance/1000;
	}
	
	/**
	 * Unify marker for hybrid
	 * @param taxon String for taxon
	 * @return String
	 */
	public String rewriteHybridMarker(String taxon){
		String taxon2 = taxon;
		Pattern p1 = Pattern.compile("^.*(\\s)x(\\s).*$");
		Matcher m1 = p1.matcher(taxon);
		if(m1.matches()){
			taxon2 = taxon.replaceFirst(" x ", " × ");
		}
		Pattern p2 = Pattern.compile("^.*(\\s)×(\\w).*$");
		Matcher m2 = p2.matcher(taxon);
		if(m2.matches()){
			taxon2 = taxon.replaceFirst("×", "× ");
		}
		return taxon2;
	}
	
	/**
	 * Cange more space to one (max 4 space)
	 * @param val Input value
	 * @return String
	 */
	public String delMoreSpace(String val){
		String value = val.trim();
		value = value.replace("  ", " ");
		value = value.replace("  ", " ");
		value = value.replace("  ", " ");
		return value.trim();
	}
	
	public String[] prepareCf(String taxon){
		String[] taxon2 = new String[2];
		taxon2[0] = null;
		taxon2[1] = null;
		Pattern p1 = Pattern.compile("^.*(\\s)cf.(\\s).*$");
		Matcher m1 = p1.matcher(taxon);
		if(m1.matches()){
			taxon2[0] = taxon.replaceFirst(" cf. ", " ");
			taxon2[0] = delMoreSpace(taxon2[0]);
			taxon2[1] = "cf.";
		}
		
		return taxon2;
	}
}
