package org.hansv.vratickaimport3;

import java.io.IOException;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Scene;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TablePosition;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.TextFieldTableCell;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.util.Callback;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.IMapTable;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public abstract class BaseController implements Initializable{
	
	@FXML
	protected TableView table_data;
	@FXML
	protected ComboBox combo_value;
	
	protected Map<String,Integer> columnsNames = new HashMap<>();
	protected ErrorLog errlog;
	protected Messages msg;
	protected IMapTable model;
	protected ObservableList<ObservableList> data;
	protected ArrayList comboData;
	protected DbUtils du;
	protected Utils utils;
	protected ObservableList list;

	
	@Override
	public abstract void initialize(URL url, ResourceBundle rb);

	protected void setTableColumns(){
		ResultSet rs = model.getData();
		table_data.getColumns().clear();
		List<String> columns = new ArrayList<>();
		try{
			ResultSetMetaData rsmd = rs.getMetaData();
			int ii = 0;
			while(ii< rsmd.getColumnCount()){
				ii++;
				columns.add(rsmd.getColumnLabel(ii));
			};
			int i = 0;
			for(String columnName : columns) {
				final int j = i;
				TableColumn col = new TableColumn(columnName);
				col.setCellValueFactory(new Callback<TableColumn.CellDataFeatures<ObservableList,String>,ObservableValue<String>>(){
					public ObservableValue<String> call(TableColumn.CellDataFeatures<ObservableList, String> param) {
						return new SimpleStringProperty(param.getValue().get(j).toString());
					}
				});
				if(columnName.equals("ID")){
					col.getStyleClass().add("columnid");
				}else{
					col.setCellFactory(TextFieldTableCell.forTableColumn());
				}
				columnsNames.put(columnName,i);
				table_data.getColumns().addAll(col);
				i++;
			}
		}catch(SQLException e){
			String descr = "Nezdařilo se nastavit sloupce pro tabulku";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	public void loadData(){
		ResultSet rs = model.getData();
		int recordCount;
		try{
			ResultSetMetaData rsmd = rs.getMetaData();
			rs.last();
			recordCount = rs.getRow();
			rs.first();
			data = FXCollections.observableArrayList();
			for(int i = 0; i < recordCount; i++){
				ObservableList<String> row = FXCollections.observableArrayList();
				for(int j = 1;j <= rsmd.getColumnCount();j++ ){
					if(StringUtils.isBlank(rs.getString(j))){
						row.add("");
					}else{
						row.add(rs.getString(j));
					}
				}
				data.add(row);
				rs.next();
			}
			table_data.setItems(data);
		}catch(SQLException e){
			e.printStackTrace();
			String descr = "Nezdařilo se získání dat z tabulky";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	
	protected void clearData(){
		String[] itemdata = new String[3];
		if(data == null){
			msg.showWarn("Není co mazat", "Načtěte data do tabulky tlačítkem \n\"Zobrazení dat\"");
			return;
		}
		for(int i = 0; i< data.size();i++){
			ObservableList row = data.get(i);
			itemdata[0] = row.get(1).toString();
			itemdata[1] = null;
			itemdata[2] = null;
			row.set(2, "");
			row.set(3, "");
			data.set(i, row);
			model.save(itemdata);
		}
	}
	
	protected void add(String path){
		try {
			FXMLLoader loader = new FXMLLoader(getClass().getResource("/fxml/WWWBrowser.fxml"));
			Stage stage = new Stage(StageStyle.DECORATED);
			stage.setScene(new Scene((Pane) loader.load()));
			WWWBrowserController controller = loader.<WWWBrowserController>getController();
			controller.load(path);
			stage.show();
			
		} catch (IOException e) {
			String descr = "Nezdařilo se otevřít okno s vestavěným WWW prohlížečem";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	protected void delete(){
		ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
		if(selectedCells.isEmpty()){
			msg.showWarn("Nemáte vybrané, co chcete pročistit", "Vyberte v tabulce, co chcete smazat");
			return;
		}
		TablePosition tablePosition = (TablePosition) selectedCells.get(0);
		ObservableList row = data.get(tablePosition.getRow());
		row.set(2, "");
		row.set(3, "");
		data.set(tablePosition.getRow(), row);
		String[] rowdata = new String[3];
		rowdata[0] = row.get(1).toString();
		rowdata[1] = null;
		rowdata[2] = null;
		model.save(rowdata);
	}
	
	public void save(){
		Object nameObj = combo_value.getValue();
		if(nameObj == null){
			msg.showWarn("Nemáte zadanou hodnotu, kterou chcete přiřadit", "Zadejte ji ve výběrovém seznamu");
			return;
		}
		ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
		if(selectedCells.isEmpty()){
			msg.showWarn("Nemáte vybraný řádek, ke kterému chcete přiřadit hodnotu.", "Vyberte ho v tabulce");
			return;
		}
		if(utils.getKey(nameObj.toString(),comboData) == null){
			msg.showWarn("Zapsaná hodnota ve výběrovém seznamu není platná", "Vyberte jí znova");
			return;
		}
		TablePosition tablePosition = (TablePosition) selectedCells.get(0);
		ObservableList row = data.get(tablePosition.getRow());
		row.set(2, nameObj.toString());
		row.set(3, utils.getKey(nameObj.toString(),comboData));
		data.set(tablePosition.getRow(), row);
		String[] values = new String[3];
		values[0] = row.get(1).toString();
		values[1] = nameObj.toString();
		values[2] = utils.getKey(nameObj.toString(),comboData);
		model.save(values);
	}
	
	protected void combo(ArrayList comboData2){
		String text = combo_value.getEditor().getText();
		comboData = comboData2;
		list = FXCollections.observableArrayList();
		for(int i = 0; i<comboData.size(); i++){
			String[] value = (String[])comboData.get(i);
			list.add(value[1]);;
		}
		combo_value.setItems(list);
		combo_value.show();
		combo_value.getEditor().setText(text);
		combo_value.getEditor().end();
	}
}
