
package org.hansv.vratickaimport3;


import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TablePosition;
import javafx.scene.control.TableView;
import javafx.scene.control.Tooltip;
import javafx.scene.control.cell.TextFieldTableCell;
import javafx.scene.input.KeyCode;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.util.Callback;
import org.hansv.model.CollectorModel;
import org.hansv.model.RestModel;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class CollectorController extends BaseController{

	@FXML
	private Button reload_collector_data;
	@FXML
	private Button clear_collector;
	@FXML
	private Button save_collectors;
	@FXML
	private Button decomp_collectors;
	@FXML
	private Button delete_collectors;
	@FXML
	private Button delete_collector;
	@FXML
	private Button add_collector;
	@FXML
	private Button save_collector;
	@FXML
	private TableView table_collectors;
	@FXML
	private ComboBox collector_delim;
	@Inject
	WWWBrowserController wbc;
	


	private ObservableList<ObservableList> dataCollectors;
	private Map<String,Integer> columnsNames2 = new HashMap<>();
	private RestModel rm;

	/**
	 * Initializes the controller class.
	 */
	@Override
	public void initialize(URL url, ResourceBundle rb) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		model = (CollectorModel)context.getBean("collectorModel");
		rm = (RestModel)context.getBean("restModel");
		du = (DbUtils)context.getBean("dbUtils");
		utils = (Utils)context.getBean("utils");
		utils.setUp();
		if(du.tableExist("COLLECTOR") == false){
			reload_collector_data.setDisable(true);
		}
		Tooltip tp1 = new Tooltip();
		tp1.setText("Zobrazení dat");
		reload_collector_data.setTooltip(tp1);
		Tooltip tp2 = new Tooltip();
		tp2.setText("Zrušení přiřazení nálezců");
		clear_collector.setTooltip(tp2);
		Tooltip tp3 = new Tooltip();
		tp3.setText("Načíst nálezce");
		save_collectors.setTooltip(tp3);
		Tooltip tp4 = new Tooltip();
		tp4.setText("Přiřadit nálezce");
		decomp_collectors.setTooltip(tp4);
		Tooltip tp5 = new Tooltip();
		tp5.setText("Smazat přiřazení nálezců");
		delete_collectors.setTooltip(tp5);
		Tooltip tp6 = new Tooltip();
		tp6.setText("Smazat nálezce");
		delete_collector.setTooltip(tp6);
		Tooltip tp7 = new Tooltip();
		tp7.setText("Nový nálezce");
		add_collector.setTooltip(tp7);
		Tooltip tp8 = new Tooltip();
		tp8.setText("Přiřadit nálezce");
		save_collector.setTooltip(tp8);
		combo_value.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>(){
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if(combo_value.getEditor().getText().length() > 2 && t.getCode().isArrowKey() == false && t.getCode() != KeyCode.ENTER){
					comboCollectorThread(combo_value.getEditor().getText());
				}
			}
		});
		combo_value.getEditor().focusedProperty().addListener(
			(observable, oldValue, newValue) -> {
				if (newValue) {
					Platform.runLater(combo_value.getEditor()::selectAll);
				}
			}
		);
		
		table_data.setOnMouseClicked(new EventHandler<MouseEvent>() {
			@Override
			public void handle(MouseEvent mouseEvent) {
				if(mouseEvent.getButton().equals(MouseButton.PRIMARY)){
					if(mouseEvent.getClickCount() == 2){
						setCollectors();
					}
				}
			}
		});
		handleReload();
	}
	
	
	
	@FXML
	public void handleAdd(ActionEvent event){
		add("person/editremote");
	}
	
	@FXML
	public void handleDecompCollectors(){
		if(data == null){
			msg.showWarn("Načtěte nálezce", "Načtěte nálezce do tabulky tlačítkem Zobrazení dat");
			return;
		}
		ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
		if(selectedCells.isEmpty()){
			msg.showWarn("Nemáte vybrané přispěvatele, které chcete přiřadit", "Vyberte přispěvatele v horní tabulce");
			return;
		}
		setCollectors();
			
	}
	
	@FXML
	public void handleReload(){
		if(du.tableExist("COLLECTOR")){
			setTableColumns();
			loadData();
		}
	}
	
	@FXML
	public void handleSave(){
		saveCollector();
	}
	
	
	@FXML
	public void handleDelete(){
		delete();
	}
	
	@FXML
	public void handleSaveCollectors(){
		saveCollectors();
	}
	
	@FXML
	public void handleClear(){
		clearData();
	}
	
	@FXML
	public void handleDeleteCollectors(){
		deleteCollectors();
	}

	
	private void deleteCollectors(){
		ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
		if(selectedCells.isEmpty()){
			msg.showWarn("Nemáte vybrané přispěvatele, které chcete vyčistit", "Vyberte přispěvatele v horní tabulce");
			return;
		}
		TablePosition tablePosition = (TablePosition) selectedCells.get(0);
		ObservableList row = data.get(tablePosition.getRow());
		row.set(2, "");
		row.set(3, "");
		data.set(tablePosition.getRow(), row);
		String[] taxondata = new String[3];
		taxondata[0] = row.get(1).toString();
		taxondata[1] = null;
		taxondata[2] = null;
		model.save(taxondata);
	}
	
	
	
	private void saveCollectors(){
		if(data == null){
			msg.showWarn("Není kam ukládat", "Načtěte přispěvatele do horní tabulky tlačítkem \n\"Zobrazení dat\"");
			return;
		}
		ObservableList selectedDataCells = table_data.getSelectionModel().getSelectedCells();
		if(selectedDataCells.isEmpty()){
			msg.showWarn("Nemáte vybrané nálezce, které chcete přiřadit", "Vyberte nálezce v horní tabulce");
			return;
		}
		
		if(dataCollectors == null){
			msg.showWarn("Není co ukládat", "Načtěte nálezce do spodní tabulky");
			return;
		}
		ObservableList selectedCollectorCells = table_collectors.getSelectionModel().getSelectedCells();
		if(selectedCollectorCells.isEmpty()){
			msg.showWarn("Nemáte vybraného nálezce, kterého chcete přiřadit", "Vyberte nálezce ve spodní tabulce");
			return;
		}
		String[] itemdata = new String[3];
		itemdata[1] = "";
		itemdata[2] = "";
		TablePosition tablePosition = (TablePosition) selectedDataCells.get(0);
		ObservableList rowData = data.get(tablePosition.getRow());
		itemdata[0] = rowData.get(1).toString();
		for(int i = 0; i< dataCollectors.size();i++){
			ObservableList rowColectors = dataCollectors.get(i);
			
			if(itemdata[1].length()>0 && rowColectors.get(1).toString().length() > 0){
				itemdata[1] += "#";
			}
			if(itemdata[2].length()>0 && rowColectors.get(2).toString().length() > 0){
				itemdata[2] += "#";
			}
			itemdata[1] += rowColectors.get(1).toString();
			itemdata[2] += rowColectors.get(2).toString();
			rowData.set(2, itemdata[1]);
			rowData.set(3, itemdata[2]);
		}
		data.set(tablePosition.getRow(), rowData);
		model.save(itemdata);
		deleteDataCollectors();
	}
	
	
	private void saveCollector(){
		Object nameObj = combo_value.getValue();
		if(utils.getKey(nameObj.toString(),comboData) == null){
			msg.showWarn("Nemáte řádně vybraného nálezce", "Vyberte nálezce ve výběrovém seznamu");
			return;
		}
		if(nameObj == null){
			msg.showWarn("Nemáte vybraného nálezce, kterého chcete přiřadit", "Vyberte nálezce ve výběrovém seznamu");
			return;
		}
		ObservableList selectedCells = table_collectors.getSelectionModel().getSelectedCells();
		if(selectedCells.isEmpty() && dataCollectors.size() >1){
			msg.showWarn("Nemáte vybraného nálezce, kterého chcete přiřadit", "Vyberte nálezce ve spodní tabulce");
			return;
		}
		int intPosition;
		if(dataCollectors.size() == 1){
			intPosition = 0;
			table_collectors.requestFocus();
			table_collectors.getSelectionModel().select(0);
			table_collectors.getFocusModel().focus(0);
		}else{
			TablePosition tablePosition = (TablePosition) selectedCells.get(0);
			intPosition = tablePosition.getRow();
		}
		
		ObservableList row = dataCollectors.get(intPosition);
		row.set(1, nameObj.toString());
		row.set(2, utils.getKey(nameObj.toString(),comboData));
		dataCollectors.set(intPosition, row);
	}
	
	
	
	private void setTableCollectorsColumn(){
		table_collectors.getColumns().clear();
		List<String> columns = new ArrayList<>();
		columns.add("Originální nálezce");
		columns.add("Mapovaný nálezce");
		columns.add("Kód");
		int i = 0;
		for(String columnName2 : columns) {
			final int j = i;
			TableColumn col = new TableColumn(columnName2);
			col.setCellValueFactory(new Callback<TableColumn.CellDataFeatures<ObservableList,String>,ObservableValue<String>>(){
				public ObservableValue<String> call(TableColumn.CellDataFeatures<ObservableList, String> param) {
					return new SimpleStringProperty(param.getValue().get(j).toString());
				}
			});
			if(columnName2.equals("ID")){
				col.getStyleClass().add("columnid");
			}else{
				col.setCellFactory(TextFieldTableCell.forTableColumn());
			}
			columnsNames.put(columnName2,i);
			table_collectors.getColumns().addAll(col);
			i++;
		}
	}
	
	private void loadDataCollectors(){
		ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
		TablePosition tablePosition = (TablePosition) selectedCells.get(0);
		ObservableList rowData = data.get(tablePosition.getRow());
		String coll = rowData.get(1).toString();
		String[] collestorArr = coll.split(collector_delim.getValue().toString());
		dataCollectors = FXCollections.observableArrayList();
		for (String collestorArr1 : collestorArr) {
			ObservableList<String> row1 = FXCollections.observableArrayList();
			row1.add(collestorArr1.trim());
			row1.add("");
			row1.add("");
			dataCollectors.add(row1);
		}
		table_collectors.setItems(dataCollectors);
	}
	
	private void deleteDataCollectors(){
		table_collectors.setItems(null);
	}
	
	private void comboCollectorThread(String sample){
		Thread t = new Thread(()->{
			Platform.runLater(new Runnable(){
				public void run(){
					combo(sample);
				}
			});
		});
		t.start();
	}
	
	private void combo(String sample){
		String text = combo_value.getEditor().getText();
		comboData = rm.getCollectors(sample);
		list = FXCollections.observableArrayList();
		for(int i = 0; i<comboData.size(); i++){
			String[] collectorArray = (String[])comboData.get(i);
			list.add(collectorArray[1]);
		}
		combo_value.setItems(list);
		combo_value.show();
		combo_value.getEditor().setText(text);
		combo_value.getEditor().end();
	}
	
	private void setCollectors(){
		setTableCollectorsColumn();
		loadDataCollectors();
	}
}
