package org.hansv.vratickaimport3;

import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.net.URL;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.ResourceBundle;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.collections.transformation.SortedList;
import javafx.concurrent.WorkerStateEvent;
import javafx.event.EventHandler;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.ListView;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.Tooltip;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeView;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.CollectorModel;
import org.hansv.model.CoordaccuracyModel;
import org.hansv.model.CoordsourceModel;
import org.hansv.model.InputImportModel;
import org.hansv.model.NamedPlaceModel;
import org.hansv.model.ProjectModel;
import org.hansv.model.SourceModel;
import org.hansv.model.TaxonModel;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class ColumnMappingController implements Initializable {
	@FXML
	public Button input_column_reload;
	@FXML
	public Button input_column_add;
	@FXML
	public Button input_column_del;
	@FXML
	public Button input_import;
	@FXML
	public ListView table_columns;
	@FXML
	private TreeView database_columns;
	@FXML
	private TextField input_value;
	@FXML
	private ProgressIndicator input_progress;
	@Inject
	LoadedDataController ldc;
	@Inject
	InputDataController idc;
	@Inject
	TaxonController tc;
	@Inject
	CollectorController cc;
	@Inject
	NamedPlaceController npc;
	@Inject
	SourceController sc;
	@Inject
	ProjectController pc;
	@Inject
	CoordsourceController csc;
	@Inject
	CoordaccuracyController cac;

	private ErrorLog errlog;
	private Messages msg;
	private ObservableList<String> listColumns;
	private SortedList<String> columnsSortedAsc;
	private List<String> columns;
	private Utils utils;
	private InputImportModel iim;
	private TaxonModel tm;
	private CollectorModel cm;
	private NamedPlaceModel npm;
	private SourceModel sm;
	private ProjectModel pm;
	private CoordsourceModel csm;
	private CoordaccuracyModel cam;

	@Override
	public void initialize(URL url, ResourceBundle rb) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		utils = (Utils)context.getBean("utils");
		iim = (InputImportModel)context.getBean("inputImportModel");
		tm = (TaxonModel)context.getBean("taxonModel");
		cm = (CollectorModel)context.getBean("collectorModel");
		sm = (SourceModel)context.getBean("sourceModel");
		pm = (ProjectModel)context.getBean("projectModel");
		npm = (NamedPlaceModel)context.getBean("namedPlaceModel");
		csm = (CoordsourceModel)context.getBean("coordsourceModel");
		cam = (CoordaccuracyModel)context.getBean("coordaccuracyModel");
		msg = (Messages)context.getBean("messages");
		Tooltip tp5 = new Tooltip();
		tp5.setText("Znovu načísts sloupce");
		input_column_reload.setTooltip(tp5);
		Tooltip tp6 = new Tooltip();
		tp6.setText("Přiřadit sloupec");
		input_column_add.setTooltip(tp6);
		Tooltip tp7 = new Tooltip();
		tp7.setText("Odebrat přiřazení sloupce");
		input_column_del.setTooltip(tp7);
		setDatabaseColumns();
		Tooltip tp8 = new Tooltip();
		tp8.setText("Import dat do pomocné tabulky");
		input_import.setTooltip(tp8);
	}


	private void setDatabaseColumns(){
		TreeItem<String> rootItem = new TreeItem<> ("Databázové sloupce");
		rootItem.setExpanded(true);
		TreeItem<String> item1 = new TreeItem<>("Taxon",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item1);
		TreeItem<String> item2 = new TreeItem<>("Originální jméno taxonu",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item2);
		TreeItem<String> item3 = new TreeItem<>("Přesnost určení",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item3);
		TreeItem<String> item5 = new TreeItem<>("Oblast",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item5);
		TreeItem<String> item6 = new TreeItem<>("Fytochorion",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item6);
		TreeItem<String> item7 = new TreeItem<>("Sídlo",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item7);
		TreeItem<String> item8 = new TreeItem<>("Stát",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item8);
		TreeItem<String> item9 = new TreeItem<>("Kvadrant",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item9);
		TreeItem<String> item10 = new TreeItem<>("Lokalita",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item10);
		TreeItem<String> item11 = new TreeItem<>("Zeměpisná šířka",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item11);
		TreeItem<String> item12 = new TreeItem<>("Zeměpisná délka",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item12);
		TreeItem<String> item13 = new TreeItem<>("Přesnost souřadnic",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item13);
		TreeItem<String> item14 = new TreeItem<>("Zdroj souřadnic",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item14);
		TreeItem<String> item15 = new TreeItem<>("Nadmořská výška",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item15);
		TreeItem<String> item16 = new TreeItem<>("Biotop",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item16);
		TreeItem<String> item17 = new TreeItem<>("Početnost",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item17);
		TreeItem<String> item18 = new TreeItem<>("Poznámka k lokalitě",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item18);
		TreeItem<String> item19 = new TreeItem<>("Poznámka k nálezu",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item19);
		TreeItem<String> item20 = new TreeItem<>("Datum1",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item20);
		TreeItem<String> item21 = new TreeItem<>("Datum2",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item21);
		TreeItem<String> item22 = new TreeItem<>("Nálezci",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item22);
		TreeItem<String> item23 = new TreeItem<>("Herbář",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item23);
		TreeItem<String> item24 = new TreeItem<>("Zdroj",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item24);
		TreeItem<String> item25 = new TreeItem<>("Projekt",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-important.png"))));
		rootItem.getChildren().add(item25);
		TreeItem<String> item26 = new TreeItem<>("Negativní nález",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item26);
		TreeItem<String> item27 = new TreeItem<>("Determinoval",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item27);
		TreeItem<String> item28 = new TreeItem<>("Revidoval",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item28);
		TreeItem<String> item29 = new TreeItem<>("Počet sched/herbářových listů",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item29);
		TreeItem<String> item30 = new TreeItem<>("Poznámka na schedě",new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-new.png"))));
		rootItem.getChildren().add(item30);
		database_columns.setRoot(rootItem);
		database_columns.setShowRoot(false);
	}

	@FXML
	private void handleInputColumnsReload(){
		database_columns.setRoot(null);
		setDatabaseColumns();
		setListTableColumns(idc.table_data);
	}

	public <T>void handleInputColumnsReload(TableView<T> tableView){
		database_columns.setRoot(null);
		setDatabaseColumns();
		setListTableColumns(tableView);
	}

	@FXML
	private void handleInputColumnAdd(){
		int listSelectedItem = table_columns.getSelectionModel().getSelectedIndex();
		TreeItem treeSelectedItem = (TreeItem)database_columns.getSelectionModel().getSelectedItem();
		String columnName;
		if(listSelectedItem == -1 && input_value.getText().length() == 0){
			msg.showWarn("Nemáte označený vstupní sloupec nebo vstupní hodnotu", "");
			return;
		}
		if(treeSelectedItem == null){
			msg.showWarn("Nemáte označený databázový sloupec", "");
			return;
		}
		if(treeSelectedItem.getParent().getValue() != "Databázové sloupce"){
			treeSelectedItem = treeSelectedItem.getParent();
		}
		if(listSelectedItem == -1){
			columnName = "'" + input_value.getText() + "'";
			input_value.setText(null);
		}else{
			columnName = columnsSortedAsc.get(listSelectedItem);
			listColumns.removeAll(columnName);
		}

		table_columns.setItems(null);
		table_columns.setItems(columnsSortedAsc);
		treeSelectedItem.getChildren().add(new TreeItem("    " + columnName));
		String dbColumnName = treeSelectedItem.getValue().toString();
		treeSelectedItem.setExpanded(true);
		String path = "/images/emblem-" + getImageFragmentName2(dbColumnName) + ".png";
			treeSelectedItem.setGraphic(new ImageView( new Image(getClass().getResourceAsStream(path))));
		checkImport();
	}

	@FXML
	private void handleInputColumnDel(){
		TreeItem treeSelectedItem = (TreeItem)database_columns.getSelectionModel().getSelectedItem();
		if(treeSelectedItem == null){
			msg.showWarn("Nemáte označený databázový sloupec", "");
			return;
		}
		if(treeSelectedItem.getParent().getValue() == "Databázové sloupce"){
			msg.showWarn("Nemáte označený databázový sloupec", "Označte položku o úroveň níže");
			return;
		}
		String columnName = treeSelectedItem.getParent().getValue().toString().trim();
		TreeItem dbItem = treeSelectedItem.getParent();
		dbItem.getChildren().remove(treeSelectedItem);
		if(dbItem.getChildren().isEmpty() == true){
			dbItem.setGraphic(new ImageView( new Image(getClass().getResourceAsStream("/images/emblem-" + getImageFragmentName(columnName) + ".png"))));
		}
		Pattern p = Pattern.compile("^'.+'$");
		Matcher m = p.matcher(treeSelectedItem.getValue().toString().substring(4).trim());
		if(m.matches()==false){
			listColumns.add(treeSelectedItem.getValue().toString().substring(4));
			table_columns.setItems(null);
			table_columns.setItems(columnsSortedAsc);
		}else{
			input_value.setText(treeSelectedItem.getValue().toString().substring(4).substring(1,treeSelectedItem.getValue().toString().substring(4).length()-1));
		}
		checkImport();
	}

	@FXML
	private void handleInputValue(){
		table_columns.getSelectionModel().clearSelection();
	}

	@FXML
	private void handleInputImport(){
		iim.createTable();
		iim.setOnFailed((event)-> {
			Throwable t = iim.getException();
			String descr = "Chyba importu do tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(t), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(t);
			msg.showErr(errs[0] + errs[1], descr);
		});
		iim.setParams(columns, idc.datacsv, database_columns.getRoot().getChildren());
		input_progress.progressProperty().bind(iim.progressProperty());
		iim.reset();
		iim.setOnSucceeded(new EventHandler<WorkerStateEvent>(){
			@Override
			public void handle(WorkerStateEvent event) {
				ldc.loadData();
				tm.createTable();
				tc.handleReload();
				cm.createTable();
				cc.handleReload();
				npm.createTable();
				npc.handleReload();
				pm.createTable();
				pc.handleReload();
				sm.createTable();
				sc.handleReload();
				cam.createTable();
				cac.handleReload();
				csm.createTable();
				csc.handleReload();
			}
		});
		iim.start();
		ldc.setTableEditable();
	}
	
	/**
	 * Get fragment name of image emblem-important.png based on column name
	 * @param column Column name
	 * @return String
	 */
	private String getImageFragmentName(String column){
		String image;
		switch(column){
			case "Taxon":
				image = "important";
				break;
			case "Lokalita":
				image = "important";
				break;
			case "Zeměpisná šířka":
				image = "important";
				break;
			case"Zeměpisná délka":
				image = "important";
				break;
			case"Přesnost souřadnic":
				image = "important";
				break;
			case "Zdroj souřadnic":
				image = "important";
				break;
			case "Datum1":
				image = "important";
				break;
			case "Nálezci":
				image = "important";
				break;
			case "Zdroj":
				image = "important";
				break;
			case "Projekt":
				image = "important";
				break;
			default:
				image = "new";
				break;
		}
		return image;
	}
	/**
	 * Get fragment name of image emblem-default.png based on column name
	 * @param column Column name
	 * @return String
	 */
	private String getImageFragmentName2(String column){
		String image;
		switch(column){
			case "Taxon":
				image = "default";
				break;
			case "Fytochorion":
				image = "default";
				break;
			case "Lokalita":
				image = "default";
				break;
			case "Zeměpisná šířka":
				image = "default";
				break;
			case"Zeměpisná délka":
				image = "default";
				break;
			case"Přesnost souřadnic":
				image = "default";
				break;
			case "Zdroj souřadnic":
				image = "default";
				break;
			case "Datum1":
				image = "default";
				break;
			case "Nálezci":
				image = "default";
				break;
			case "Zdroj":
				image = "default";
				break;
			case "Projekt":
				image = "default";
				break;
			default:
				image = "new";
				break;
		}
		return image;
	}
		
	private void checkImport(){
		ObservableList<String> treeList = database_columns.getRoot().getChildren();
		TreeItem item;
		for(Object o: treeList){
			item = (TreeItem)o;
			if(getImageFragmentName(item.getValue().toString()) == "important"){
				if(item.getChildren().isEmpty()){
					input_import.setDisable(true);
					return;
				}
			}
		}
		input_import.setDisable(false);
	}

	public <T>void setListTableColumns(TableView<T> tableView) {
		columns = new ArrayList<>();
		tableView.getColumns().stream().forEach((col) -> {
			columns.add(col.getText());
		});
		List duplicityColumns = utils.getDuplicitInList(columns);
		String descr = new String();
		descr = "";
		if(duplicityColumns.isEmpty() == false){
			for(Object o:duplicityColumns){
				if(descr.length() > 0){
					descr += "; ";
				}
				descr += o.toString();
			}
			msg.showWarn("Duplicitní sloupce",descr);
			input_column_add.setDisable(true);
			input_column_del.setDisable(true);
			input_column_reload.setDisable(true);
		}else{
			input_column_add.setDisable(false);
			input_column_del.setDisable(false);
			input_column_reload.setDisable(false);
		}
		listColumns = FXCollections.observableArrayList(columns);
		SortedList columnsSorted = new SortedList(listColumns);
		columnsSortedAsc = columnsSorted.sorted(Comparator.<String>naturalOrder());
		table_columns.setItems(columnsSortedAsc);
	}
}
