package org.hansv.vratickaimport3;

import com.google.inject.Singleton;
import java.net.URL;
import java.util.ArrayList;
import java.util.ResourceBundle;
import javafx.collections.FXCollections;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Tooltip;
import org.hansv.model.CoordaccuracyModel;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class CoordaccuracyController extends BaseController{
	

	@FXML
	private Button reload_data;
	@FXML
	private Button clear_coordaccuracy;
	@FXML
	private Button save_coordaccuracy;
	@FXML
	private Button delete_coordaccuracy;


	@Override
	public void initialize(URL location, ResourceBundle resources) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		model = (CoordaccuracyModel)context.getBean("coordaccuracyModel");
		utils = (Utils)context.getBean("utils");
		du = (DbUtils)context.getBean("dbUtils");
		Tooltip tp1 = new Tooltip();
		tp1.setText("Zobrazení dat");
		reload_data.setTooltip(tp1);
		Tooltip tp3 = new Tooltip();
		tp3.setText("Zrušení namapování");
		clear_coordaccuracy.setTooltip(tp3);
		Tooltip tp4 = new Tooltip();
		tp4.setText("Přiřazení přesnosti souřadnic");
		save_coordaccuracy.setTooltip(tp4);
		Tooltip tp5 = new Tooltip();
		tp5.setText("Zrušení mapování přesnosti souřadnic");
		delete_coordaccuracy.setTooltip(tp5);
		combo();
		handleReload();
	}
	
	@FXML
	public void handleReload(){
		if(du.tableExist("COORDACCURACY")){
			setTableColumns();
			loadData();
		}
	}
	
	@FXML
	public void handleClear(){
		clearData();
	}
	
	@FXML
	public void handleSave(){
		save();
	}
	
	@FXML
	public void handleDelete(){
		delete();
	}
	
	private void combo(){
		comboData = getComboValue();
		list = FXCollections.observableArrayList();
		for(int i = 0; i<comboData.size(); i++){
			String[] taxon = (String[])comboData.get(i);
			list.add(taxon[1]);
		}
		combo_value.setItems(list);
	}
	
	private ArrayList<String[]> getComboValue(){
		ArrayList<String[]> values = new ArrayList<>();
		values.add(new String[]{"1","Metry"});
		values.add(new String[]{"10","Desítky metrů"});
		values.add(new String[]{"100","Stovky metrů"});
		values.add(new String[]{"1000","Kilometry"});
		values.add(new String[]{"10000","Desítky kilometrů"});
		return values;
	}
	
}
