package org.hansv.vratickaimport3;


import com.google.inject.Singleton;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.ResourceBundle;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.TableView;
import javafx.scene.control.Tooltip;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.CoordsourceModel;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class CoordsourceController extends BaseController{
	@FXML
	private Button reload_data;
	@FXML
	private Button clear_coordsource;
	@FXML
	private Button save_coordsource;
	@FXML
	private Button delete_coordsource;

	
	@Override
	public void initialize(URL location, ResourceBundle resources) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		model = (CoordsourceModel)context.getBean("coordsourceModel");
		utils = (Utils)context.getBean("utils");
		du = (DbUtils)context.getBean("dbUtils");
		Tooltip tp1 = new Tooltip();
		tp1.setText("Zobrazení dat");
		reload_data.setTooltip(tp1);
		Tooltip tp3 = new Tooltip();
		tp3.setText("Zrušení namapování");
		clear_coordsource.setTooltip(tp3);
		Tooltip tp4 = new Tooltip();
		tp4.setText("Přiřazení přesnosti souřadnic");
		save_coordsource.setTooltip(tp4);
		Tooltip tp5 = new Tooltip();
		tp5.setText("Zrušení mapování přesnosti souřadnic");
		delete_coordsource.setTooltip(tp5);
		combo();
		handleReload();
	}
	
	@FXML
	public void handleReload(){
		if(du.tableExist("COORDSOURCE")){
			setTableColumns();
			loadData();
		}
	}
	
	@FXML
	public void handleClear(){
		clearData();
	}
	
	@FXML
	public void handleSave(){
		save();
	}
	
	@FXML
	public void handleDelete(){
		delete();
	}
	
	/*public void loadData(){
		ResultSet rs = model.getData();
		int recordCount;
		try{
			ResultSetMetaData rsmd = rs.getMetaData();
			rs.last();
			recordCount = rs.getRow();
			rs.first();
			data = FXCollections.observableArrayList();
			for(int i = 0; i < recordCount; i++){
				ObservableList<String> row = FXCollections.observableArrayList();
				for(int j = 1;j <= rsmd.getColumnCount();j++ ){
					if(StringUtils.isBlank(rs.getString(j))){
						row.add("");
					}else{
						row.add(rs.getString(j));
					}
				}
				data.add(row);
				rs.next();
			}
			table_coordsource.setItems(data);
		}catch(SQLException e){
			e.printStackTrace();
			String descr = "Nezdařilo se získání dat z tabulky coordsource";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}*/

	
	private void combo(){
		comboData = getComboValue();
		list = FXCollections.observableArrayList();
		for(int i = 0; i<comboData.size(); i++){
			String[] taxon = (String[])comboData.get(i);
			list.add(taxon[1]);
		}
		combo_value.setItems(list);
	}
	
	private ArrayList<String[]> getComboValue(){
		ArrayList<String[]> values = new ArrayList<>();
		values.add(new String[]{"1","Jiný původ"});
		values.add(new String[]{"2","Originální souřadnice od autora"});
		values.add(new String[]{"3","Odečteny z mapy"});
		return values;
	}
}
