
package org.hansv.vratickaimport3;

import com.google.inject.Inject;
import java.sql.ResultSet;
import java.sql.SQLException;
import javafx.beans.property.ReadOnlyObjectProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Task;
import javafx.concurrent.Worker;
import javafx.concurrent.WorkerStateEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.Cursor;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.web.WebView;
import javafx.stage.Stage;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.codehaus.plexus.personality.plexus.lifecycle.phase.Initializable;
import org.codehaus.plexus.personality.plexus.lifecycle.phase.InitializationException;
import org.hansv.model.CollectorModel;
import org.hansv.model.CoordaccuracyModel;
import org.hansv.model.CoordsourceModel;
import org.hansv.model.InputImportModel;
import org.hansv.model.LocalityModel;
import org.hansv.model.NamedPlaceModel;
import org.hansv.model.PhytochorionModel;
import org.hansv.model.ProjectModel;
import org.hansv.model.SourceModel;
import org.hansv.model.TaxonModel;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class ImportController implements Initializable{
	@FXML
	private WebView wwwcheck;
	@FXML
	private Button import_import;
	@FXML
	private Button import_check;
	@FXML
	private Button import_cancel;
	@FXML
	private Label label_import;
	@FXML
	private ProgressIndicator indicator_map_import;
	@Inject
	LoadedDataController ldc;
	@Inject
	TaxonController tc;
	@Inject
	CollectorController cc;
	@Inject
	CoordaccuracyController cac;
	@Inject
	CoordsourceController csc;
	@Inject
	NamedPlaceController npc;
	@Inject
	PhytochorionController phc;
	@Inject
	ProjectController prc;
	@Inject
	SourceController sc;
	
	private ErrorLog errlog;
	private Messages msg;
	private TaxonModel tm;
	private ProjectModel pm;
	private SourceModel sm;
	private NamedPlaceModel npm;
	private CollectorModel cm;
	private CoordsourceModel csm;
	private CoordaccuracyModel cam;
	private LocalityOccurencesImport loi;
	private PhytochorionModel phm;
	private InputImportModel iim;
	private LocalityModel lm;
	private Utils utils;
	private String html;
	
	@Override
	public void initialize() throws InitializationException {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		tm = (TaxonModel)context.getBean("taxonModel");
		pm = (ProjectModel)context.getBean("projectModel");
		sm = (SourceModel)context.getBean("sourceModel");
		npm = (NamedPlaceModel)context.getBean("namedPlaceModel");
		cm = (CollectorModel)context.getBean("collectorModel");
		csm = (CoordsourceModel)context.getBean("coordsourceModel");
		cam = (CoordaccuracyModel)context.getBean("coordaccuracyModel");
		loi = (LocalityOccurencesImport)context.getBean("localityoccurencesImport");
		iim = (InputImportModel)context.getBean("inputImportModel");
		phm = (PhytochorionModel)context.getBean("phytochorionModel");
		lm = (LocalityModel)context.getBean("localityModel");
		utils = (Utils)context.getBean("utils");
		import_import.setDisable(true);
		import_cancel.setDisable(true);
		wwwcheck.getEngine().loadContent(utils.getHtmlHeader() + utils.getHtmlFoot());
		wwwcheck.getEngine().reload();
	}
	
	@FXML
	public void handleCheck(){
		Task task = new Task() {
			@Override
			protected Object call() throws Exception{
				Stage stage = (Stage)import_check.getScene().getWindow();
				stage.getScene().setCursor(Cursor.WAIT);
				import_check.setDisable(true);
				int taxon = tm.getUnmapped();
				boolean taxon2 = tm.isAnalyzed();
				int project = pm.getUnmapped();
				boolean project2 = pm.isAnalyzed();
				int source = sm.getUnmapped();
				boolean source2 = sm.isAnalyzed();
				int place = npm.getUnmapped();
				boolean place2 = npm.isAnalyzed();
				int collector = cm.getUnmapped();
				boolean collector2 = cm.isAnalyzed();
				int coordsource = csm.getUnmapped();
				boolean coordsource2 = csm.isAnalyzed();
				int coordaccuracy = cam.getUnmapped();
				boolean coordaccuracy2 = csm.isAnalyzed();
				if(taxon == 0 && project == 0 && source == 0 && place == 0 && collector == 0 && coordsource == 0 && coordaccuracy == 0 && taxon2 == true && project2 == true && source2 == true && collector2 == true && coordsource2 == true && coordaccuracy2 == true){
					createLocalities();
					setId();
					import_import.setDisable(false);
					import_cancel.setDisable(true);
				}else{
					import_import.setDisable(true);
					import_cancel.setDisable(true);
					import_check.setDisable(false);
				}
				html = setResultCheck(taxon, project, source, place, collector, coordsource, coordaccuracy, taxon2, project2, source2, place2, collector2, coordsource2, coordaccuracy2);
				stage.getScene().setCursor(Cursor.DEFAULT);
				return "Gone";
			}
		};
		new Thread(task).start();
		task.setOnSucceeded(new EventHandler<WorkerStateEvent>() {
			@Override
			public void handle(WorkerStateEvent event){
				showHtml(html);
			}
		});
		task.setOnFailed(new EventHandler<WorkerStateEvent>() {
			@Override
			public void handle(WorkerStateEvent event){
				String descr = "Exception, ImportController.handleCheck";
				errlog.logErr(ExceptionUtils.getStackTrace(task.getException()), descr);
				Stage stage = (Stage)import_check.getScene().getWindow();
				stage.getScene().setCursor(Cursor.DEFAULT);
				String[] errs = ExceptionUtils.getRootCauseStackTrace(task.getException());
				msg.showErr(errs[0] + errs[1], descr);
			}
		});
		
		
		
	}
	
	@FXML
	public void handleImport(){
		import_check.setDisable(true);
		import_import.setDisable(true);
		import_cancel.setDisable(false);
		importLocalitiesOccurences();
	}
	
	@FXML
	public void handleCancel(){
		if(loi.isRunning()){
			loi.cancel();
		}
		import_cancel.setDisable(true);
		import_import.setDisable(true);
		import_check.setDisable(false);
	}
	
	private void importLocalitiesOccurences(){
		loi.setOnFailed((event)-> {
			Throwable t = loi.getException();
			String descr = loi.getMessage();
			loi.cancel();
			wwwcheck.getEngine().loadContent(utils.getHtmlHeader()  + "<div style='color:red; font-family:sans-serif'>"+ descr + "</div>" + utils.getHtmlFoot());
			errlog.logErr(ExceptionUtils.getStackTrace(t), descr);
			import_check.setDisable(false);
			import_import.setDisable(false);
			import_cancel.setDisable(true);
			msg.showErr(descr.replace("<br />", "\n"),"Chyba importu");
		});
		indicator_map_import.progressProperty().bind(loi.progressProperty());
		label_import.textProperty().bind(loi.titleProperty());
		loi.restart();
		final ReadOnlyObjectProperty<Worker.State> state = loi.stateProperty();
		state.addListener(new ChangeListener<Worker.State>() {
			@Override
			public void changed(ObservableValue<? extends Worker.State> observable, Worker.State oldValue, Worker.State newValue) {
				if(newValue.toString().equals("SUCCEEDED")){
					if(loi.getValue().equals("Completed")){
						clearDb();
						reloadTables();
						msg.showInfo("Konec importu", "Import byl úspěšný");
						import_check.setDisable(false);
						import_import.setDisable(true);
						import_cancel.setDisable(true);
					}else{
						wwwcheck.getEngine().loadContent(utils.getHtmlHeader()  +loi.getValue() + utils.getHtmlFoot());
						loi.cancel();
						import_check.setDisable(false);
						import_cancel.setDisable(false);
						lm.deleteImportId();
						iim.deleteLocalityId();
						msg.showErr("Chyba importu lokalit", "Chyba importu");
					}
				}
			}
		});

	}

	
	private String setResultCheck(int taxon,int project, int source, int place, int collector, int coordsource, int coordaccuracy, boolean taxon2, boolean project2, boolean source2, boolean place2, boolean collector2, boolean coordsource2, boolean coordaccuracy2){
		String result =  "";
		String result2 = "";
		if (taxon >0){
			result2 += "Není přiřazeno <b>"+ taxon + " taxonů</b><br/>";
		}
		if (project > 0){
			result2 += "Není přiřazeno <b>"+ project + " projektů</b><br/>";
		}
		if (source > 0){
			result2 += "Není přiřazeno <b>"+ source + " zdrojů</b><br/>";
		}
		if (place > 0){
			result2 += "Není přiřazeno <b>"+ place + " sídel</b><br/>";
		}
		if (collector > 0){
			result2 += "Není přiřazeno <b>"+ collector + " nálezců</b><br/>";
		}
		if(coordsource > 0){
			result2 += "Není přiřazeno <b>"+ coordsource + " zdrojů souřadnic</b><br/>";
		}
		if(coordaccuracy > 0){
			result2 += "Není přiřazeno <b>"+ coordaccuracy + " přesností souřadnic</b><br/>";
		}
		if (taxon2 == false){
			result2 += "Není zaplněna tabulka <b>taxonů</b><br/>";
		}
		if (project2 == false){
			result2 += "Není zaplněna tabulka <b>projektů</b><br/>";
		}
		if (source2 == false){
			result2 += "Není zaplněna tabulka <b>zdrojů</b><br/>";
		}
		/*if (place2 == false){
			result2 += "Není zaplněna tabulka <b>sídel</b><br/>";
		}*/
		if (collector2 == false){
			result2 += "Není zaplněna tabulka <b>nálezců</b><br/>";
		}
		if(coordsource2 == false){
			result2 += "Není zaplněna tabulka <b>zdrojů souřadnic</b><br/>";
		}
		if(coordaccuracy2 == false){
			result2 += "Není zaplněna tabulka <b>přesností souřadnic</b><br/>";
		}
		if(result2.length()== 0){
			result = "<div style='font-family:sans-serif;font-size:12px;color:green'>";
			result2 ="Všechny hodoty jsou namapovány. <b>Import může být proveden</b>";
		}else{
			result = "<div style='font-family:sans-serif;font-size:12px;color:red'>";
		}
		result += result2;
		result += "</div>";
		return result;
	}
	
	private void showHtml(String html){
		wwwcheck.getEngine().loadContent(utils.getHtmlHeader() + html + utils.getHtmlFoot());
		wwwcheck.getEngine().reload();
	}
	
	private void clearDb(){
		iim.createTable();
		tm.createTable();
		cm.createTable();
		sm.createTable();
		npm.createTable();
		pm.createTable();
		phm.createTable();
		csm.createTable();
		cam.createTable();
		try{
			lm.dropTableLocalities();
		}catch(SQLException e){
			String descr = "Nezdařilo vytvoření tabulky localities";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	private void reloadTables(){
		ldc.handleReload();
		tc.handleReload();
		cc.handleReload();
		cac.handleReload();
		csc.handleReload();
		npc.handleReload();
		phc.handleReload();
		prc.handleReload();
		sc.handleReload();
	}
	
	private void setId() throws SQLException{
		ResultSet taxon = tm.getData();
		ResultSet source = sm.getData();
		ResultSet project = pm.getData();
		ResultSet collectors = cm.getData();

		while(taxon.next()){
			iim.setTaxonId(taxon.getString("orig"), taxon.getString("mapids"));
		}
		taxon.close();
		while(source.next()){
			iim.setSourceId(source.getString("orig"), source.getString("mapids"));
		}
		source.close();
		while(project.next()){
			iim.setProjectId(project.getString("orig"), project.getString("mapids"));
		}
		project.close();
		while(collectors.next()){
			iim.setCollectorsId(collectors.getString("orig"), collectors.getString("mapids"));
		}
		collectors.close();
	}
	
	private void createLocalities() throws SQLException{
		lm.dropTableLocalities();
		lm.createTableLocalities();
		lm.importLocalities();
	}
}
