package org.hansv.vratickaimport3;

import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.Cursor;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.Tooltip;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import javafx.util.Callback;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.db.Csv;
import org.hansv.utils.Configurable;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class InputDataController implements Initializable, Configurable {

	@FXML
	public TableView table_data;
	@FXML
	private ComboBox charset;
	@FXML
	private ComboBox column_delimiter;
	@FXML
	private ComboBox text_wrapper;
	@FXML
	private Button file_open;
	@FXML
	private Button reload;  
	@FXML
	private CheckBox table_header;
	@Inject
	ColumnMappingController cmc;

	private ErrorLog errlog;
	private Messages msg;
	private Csv csvdata;
	private String path;
	private ObservableList<ObservableList> data;
	public Object[][] datacsv;

	/**
	 * Initializes the controller class.
	 */
	@Override
	public void initialize(URL url, ResourceBundle rb) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		csvdata = (Csv)context.getBean("csv");
		msg = (Messages)context.getBean("messages");
		Tooltip tp1 = new Tooltip();
		tp1.setText("Otevřít tabulku");
		file_open.setTooltip(tp1);
		Tooltip tp4 = new Tooltip();
		tp4.setText("Znovu načíst tabulku");
		reload.setTooltip(tp4);
		table_header.setSelected(true);
		reload.setDisable(true);
	}

	@Override
	public void configure(){
		cmc.input_column_add.setDisable(true);
		cmc.input_column_del.setDisable(true);
		cmc.input_column_reload.setDisable(true);
		cmc.input_import.setDisable(true);
	}

	@FXML
	private void handleOpenFileButtonAction(ActionEvent event) {
		FileChooser fileChooser = new FileChooser();
		fileChooser.setTitle("Oteřít");
		fileChooser.getExtensionFilters().addAll(
			new FileChooser.ExtensionFilter("CSV","*.csv"),
			new FileChooser.ExtensionFilter("Všechny soubory","*.*")
		);
		File file = fileChooser.showOpenDialog(new Stage());
		try{
			if(file != null){
				path = file.getAbsolutePath();
				Scene s = file_open.getScene();
				s.setCursor(Cursor.WAIT);
				Thread t = new Thread(()-> {
					Platform.runLater(new Runnable(){
						public void run(){
							loadData();
							cmc.handleInputColumnsReload(table_data);
							s.setCursor(Cursor.DEFAULT);
						}
					});
				});
				t.start();
				reload.setDisable(false);
			}
		}catch(Exception e){
			String descr = "Chyba programu";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}

	private void loadData(){
		table_data.getColumns().clear();
		List<String> columns = new ArrayList<>();
		try{
			csvdata.setUp(path, getDelimiter(column_delimiter.getValue().toString()), text_wrapper.getValue().toString(), charset.getValue().toString(), table_header.selectedProperty().getValue());
			String[] columnscsv = csvdata.getColumns();
			for(String namecol:columnscsv){
				columns.add(namecol);
			}
			int i = 0;
			for(String columName : columns) {
				final int j = i;
				TableColumn col = new TableColumn(columName);
				col.setCellValueFactory(new Callback<TableColumn.CellDataFeatures<ObservableList,String>,ObservableValue<String>>(){
					public ObservableValue<String> call(TableColumn.CellDataFeatures<ObservableList, String> param) {
						return new SimpleStringProperty(param.getValue().get(j).toString());
					}
				});
				table_data.getColumns().addAll(col);
				i++;
			}
			data = FXCollections.observableArrayList();
			datacsv = csvdata.readData();
			//int ii = 0;
			for(Object[] rowcsv:datacsv){
				//Iterate Row
				ObservableList<String> row = FXCollections.observableArrayList();
				for(Object cellvalue: rowcsv){
					row.add(cellvalue.toString());
				}
				data.add(row);
			}
			table_data.setItems(data);

		}catch(Exception e){
			String descr = "Chyba programu";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}


	private String getDelimiter(String delimiterName){
		String delim;
		switch(delimiterName){
			case("mezera"): delim = " ";
				break;
			case("TAB"): delim = "\t";
				break;
			default: delim = delimiterName;
				break;
		}
		return delim;
	}

	@FXML
	private void handleReloadButtonAction(ActionEvent event) {
		Scene s = reload.getScene();
		s.setCursor(Cursor.WAIT);
		Thread t = new Thread(()-> {
			Platform.runLater(new Runnable(){
				public void run(){
					loadData();
					cmc.handleInputColumnsReload(table_data);
					s.setCursor(Cursor.DEFAULT);
				}
			});
		});
		t.start(); 
	}
}
