package org.hansv.vratickaimport3;

import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Task;
import javafx.concurrent.WorkerStateEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.Cursor;
import javafx.scene.control.Button;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableColumn.CellDataFeatures;
import javafx.scene.control.TableColumn.CellEditEvent;
import javafx.scene.control.TableView;
import javafx.scene.control.Tooltip;
import javafx.scene.control.cell.TextFieldTableCell;
import javafx.scene.web.WebView;
import javafx.stage.Stage;
import javafx.util.Callback;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.CollectorModel;
import org.hansv.model.CoordaccuracyModel;
import org.hansv.model.CoordsourceModel;
import org.hansv.model.InputImportModel;
import org.hansv.model.NamedPlaceModel;
import org.hansv.model.PhytochorionModel;
import org.hansv.model.ProjectModel;
import org.hansv.model.RestModel;
import org.hansv.model.SourceModel;
import org.hansv.model.TaxonModel;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class LoadedDataController implements Initializable{
	@FXML
	private Button check_data;
	@FXML
	private Button reload_data;
	@FXML
	private WebView check_result;
	@FXML
	private TableView table_input;
	@FXML
	private Button separate_data;
	@FXML
	private Button delete_duplicity;
	@Inject
	private TaxonController tc;
	@Inject
	private CollectorController cc;
	@Inject
	private ProjectController pc;
	@Inject
	private SourceController sc;
	@Inject
	private NamedPlaceController npc;
	@Inject
	private PhytochorionController phc;
	@Inject
	private CoordaccuracyController cac;
	@Inject
	private CoordsourceController csc;
	
	private ErrorLog errlog;
	private Messages msg;
	private InputImportModel iim;
	private ObservableList<ObservableList> data;
	private Map<String,Integer> columnsNames = new HashMap<>();
	private TaxonModel tm;
	private CollectorModel cm;
	private ProjectModel pm;
	private RestModel rm;
	private SourceModel sm;
	private NamedPlaceModel npm;
	private PhytochorionModel phm;
	private CoordaccuracyModel cam;
	private CoordsourceModel csm;
	private DbUtils du;
	private Utils utils;
	

	@Override
	public void initialize(URL url, ResourceBundle rb) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		iim = (InputImportModel)context.getBean("inputImportModel");
		tm = (TaxonModel)context.getBean("taxonModel");
		cm = (CollectorModel)context.getBean("collectorModel");
		pm = (ProjectModel)context.getBean("projectModel");
		rm = (RestModel)context.getBean("restModel");
		sm = (SourceModel)context.getBean("sourceModel");
		npm = (NamedPlaceModel)context.getBean("namedPlaceModel");
		phm = (PhytochorionModel)context.getBean("phytochorionModel");
		cam = (CoordaccuracyModel)context.getBean("coordaccuracyModel");
		csm = (CoordsourceModel)context.getBean("coordsourceModel");
		du = (DbUtils)context.getBean("dbUtils");
		utils = (Utils)context.getBean("utils");
		Tooltip tp1 = new Tooltip();
		tp1.setText("Zobrazení dat");
		reload_data.setTooltip(tp1);
		Tooltip tp2 = new Tooltip();
		tp2.setText("Kontrola dat");
		check_data.setTooltip(tp2);
		Tooltip tp3 = new Tooltip();
		tp3.setText("Rozdělení dat");
		Tooltip tp4 = new Tooltip();
		tp4.setText("Smazání duplicitních řádků");
		delete_duplicity.setTooltip(tp4);
		if(du.tableExist("INPUT") == false){
			iim.createTable();
		}
		separate_data.setTooltip(tp3);
		separate_data.setDisable(true);
		delete_duplicity.setDisable(true);
		handleReload();
		showCheckResult("");
	}

	@FXML
	public void handleReload(){
		if(du.tableExist("INPUT")){
			setTableColumns();
			loadData();
		}
	}
	
	@FXML
	public void handleDeleteDuplicity(){
		Task task = new Task() {
			@Override
			protected Object call() throws Exception {
				Stage stage = (Stage)check_data.getScene().getWindow();
				stage.getScene().setCursor(Cursor.WAIT);
				iim.deleteDuplicity();
				String result = checkData();
				updateMessage(result);
				stage.getScene().setCursor(Cursor.DEFAULT);
				return "Gone";
			}
		};
		new Thread(task).start();
		task.setOnSucceeded(new EventHandler<WorkerStateEvent>() {
			@Override
			public void handle(WorkerStateEvent event) {
				handleReload();
				showCheckResult(task.getMessage().toString());
			}
		});
	}

	@FXML
	public void handleCheckData(){
		Task task = new Task() {
			@Override
			protected Object call() throws Exception {
				Stage stage = (Stage)check_data.getScene().getWindow();
				stage.getScene().setCursor(Cursor.WAIT);
				String result = checkData();
				updateMessage(result);
				stage.getScene().setCursor(Cursor.DEFAULT);
				return "Gone";
			}
		};
		new Thread(task).start();
		task.setOnSucceeded(new EventHandler<WorkerStateEvent>() {
			@Override
			public void handle(WorkerStateEvent event) {
				handleReload();
				showCheckResult(task.getMessage().toString());
			}
		});
	}

	private void setTableColumns(){
		ResultSet rs = iim.getData();
		table_input.getColumns().clear();
		List<String> columns = new ArrayList<>();
		try{
			ResultSetMetaData rsmd = rs.getMetaData();
			int ii = 0;
			while(ii< rsmd.getColumnCount()){
				ii++;
				columns.add(rsmd.getColumnLabel(ii));
			};
			int i = 0;
			for(String columnName : columns) {
				final int j = i;
				TableColumn col = new TableColumn(columnName);
				col.setCellValueFactory(new Callback<CellDataFeatures<ObservableList,String>,ObservableValue<String>>(){
					public ObservableValue<String> call(CellDataFeatures<ObservableList, String> param) {
						return new SimpleStringProperty(param.getValue().get(j).toString());
					}
				});
				if(columnName.equals("ID_INPUT")){
					col.getStyleClass().add("columnid");
					col.setEditable(false);
				}
				if(columnName.equals("Datum 1")){
					col.getStyleClass().add("noedit");
					col.setEditable(false);
				}
				if(columnName.equals("DATUM 1")){
					col.getStyleClass().add("noedit");
					col.setEditable(false);
				}
				if(columnName.equals("ZEMĚPISNÁ ŠÍŘKA")){
					col.getStyleClass().add("noedit");
					col.setEditable(false);
				}
				if(columnName.equals("ZEMĚPISNÁ DÉLKA")){
					col.getStyleClass().add("noedit");
					col.setEditable(false);
				}
				if(columnName.equals("VZDÁŮENOST LOKALITY OD SÍDLA")){
					col.getStyleClass().add("noedit");
					col.setEditable(false);
				}
				
				if(columnName.equals("DATUM 2")){
					col.getStyleClass().add("noedit");
					col.setEditable(false);
				}else{
					col.setCellFactory(TextFieldTableCell.forTableColumn());
					col.setOnEditCommit(new EventHandler<CellEditEvent>(){
						@Override
						public void handle(CellEditEvent event) {
							saveChangeValue(event);
						}
					});
				}
				columnsNames.put(columnName,i);
				table_input.getColumns().addAll(col);
				i++;
			}
		}catch(SQLException e){
			String descr = "Nezdařilo se nastavit sloupce pro tabulku input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}

	private void saveChangeValue(CellEditEvent event){
		ObservableList row = data.get(event.getTablePosition().getRow());
		row.set(columnsNames.get(event.getTableColumn().getText()), event.getNewValue());
		data.set(event.getTablePosition().getRow(), row);
		String res = checkValue(event.getTableColumn().getText(), event.getNewValue().toString());
		if(StringUtils.isBlank(res)){
			iim.saveValue(row.get(0).toString(), event.getNewValue().toString(), event.getTableColumn().getText());
		}else{
			msg.showWarn(null, res);
		}
	}
	
	/**
	 * Check value if correspond with column type
	 * @param column
	 * @param value
	 * @return 
	 */
	private String checkValue(String column, String value){
		String res = null;
		String type = du.getColumnType("INPUT", iim.getColumnName(column).toUpperCase());

		switch(column){
			case "STÁT":
				ArrayList<String[]> cntr = rm.isCountryCode(value);
				if(cntr.get(0)[0].equals("##")){
					res = value + " není dvoupísmenná zkratka státu dle ISO 3166-1";
				}
				break;
			case "NEGATIVNÍ NÁLEZ":
				if(utils.isNegative(value) == false){
					res = value + " není celé číslo v rozmezí 0-1 nebo logická hodnota true/fals";
				}
				break; 
			case "ORIGINÁLNÍ ZEMĚPISNÁ ŠÍŘKA":
				if(utils.isDouble(value) == false){
					res = value + " není desetinné číslo";
				}
				break;
			case "ORIGINÁLNÍ ZEMĚPISNÁ DÉLKA":
				if(utils.isDouble(value) == false){
					res = value + " není desetinné číslo";
				}
				break;
			case "ORIGINÁLNÍ DATUM 1":
				if(StringUtils.isBlank(utils.transformDate(value))){
					res = value + " není datum";
				}
				break;
			case "ORIGINÁLNÍ DATUM 2":
				if(StringUtils.isBlank(utils.transformDate(value))){
					res = value + " není datum";
				}
				break;
			}
			
		return res;
	}

	public void loadData(){
		ResultSet rs = iim.getData();
		int recordCount;
		try{
			ResultSetMetaData rsmd = rs.getMetaData();
			rs.last();
			recordCount = rs.getRow();
			rs.first();
			data = FXCollections.observableArrayList();
			for(int i = 0; i < recordCount; i++){
				ObservableList<String> row = FXCollections.observableArrayList();
				for(int j = 1;j <= rsmd.getColumnCount();j++ ){
					if(StringUtils.isBlank(rs.getString(j))){
						row.add("");
					}else{
						row.add(rs.getString(j));
					}
				}
				data.add(row);
				rs.next();
			}
			table_input.setItems(data);
		}catch(SQLException e){
			e.printStackTrace();
			String descr = "Nezdařilo se získání dat z tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	public String checkData(){
		ResultSet rs = iim.getData();
		String result = "";
		String result2  = "";
		int id;
		String altitude;
		String taxon;
		String origTaxon;
		String abbrev;
		String area;
		String namedplace;
		String countryCode;
		String netmaparea;
		String locality;
		String latitude;
		String longitude;
		String coordaccu;
		String coordsrc;
		String biotope;
		String abund;
		String locnote;
		String occnote;
		String date1;
		String date2;
		String collectors;
		String herb;
		String source;
		String project;
		String negative;
		String det;
		String rev;
		String schedacount;
		ArrayList <String[]> country;
		int countErrRow = 0;
		int countErr = 0;
		int countSumErr = 0;
		
		try{
			rs.last();
			int recordCount = rs.getRow();
			rs.first();
			result2 ="<div style='font-family:sans-serif;font-size:12px'>";
			for(int i = 0; i < recordCount; i++){
				id = rs.getInt("ID_INPUT");
				taxon = rs.getString("TAXON");
				if(StringUtils.isBlank(taxon)){
					result = " chybí zadný <b>TAXON</b>";
					countErr++;
				}
				origTaxon = rs.getString("ORIGINALTAXON");
				if(StringUtils.isBlank(origTaxon) == false){
					if(origTaxon.length()>100){
						result += addDelim(result);
						result += "<b>ORIGINÁLNÍ JMÉNO TAXONU</b> je delší než 100 znaků";
						countErr++;
					}
				}
				abbrev = rs.getString("ABBREV");
				if(StringUtils.isBlank(abbrev) == false){
					if(abbrev.length()>10){
						result += addDelim(result);
						result += "zkratka <b>PŘESNOST URČENÍ</b> je delší než 10 znaků";
						countErr++;
					}
				}
				area = rs.getString("AREA");
				if(StringUtils.isBlank(area) == false){
					if(area.length() > 75){
						result += addDelim(result);
						result += "<b>OBLAST</b> je delší než 50 znaků";
						countErr++;
					}
				}
				netmaparea = rs.getString("NETMAPAREA");
				if(StringUtils.isBlank(netmaparea) == false){
					if(netmaparea.length() >7){
						result += addDelim(result);
						result = "hodnota pro pole <b>KVADRANT</b> je delší než 7 znaků";
						countErr++;
					}
				}
				locality = rs.getString("LOCALITY");
				if(StringUtils.isBlank(locality)){
					result += addDelim(result);
					result += "chybí zapsaná <b>LOKALITA</b>";
					countErr++;
				}else{
					if(locality.length() > 500){
						result += addDelim(result);
						result += "<b>LOKALITA</b> je delší než 500 znaků";
					countErr++;
					}
				}
				longitude = rs.getString("LONGITUDEORIG");
				if(StringUtils.isBlank(longitude)){
					result += addDelim(result);
					result += "chybí <b>ZEMĚPISNÁ DÉLKA</b>";
					countErr++;
				}else{
					if(utils.transformLongitude(longitude) == null){
						result += addDelim(result);
						result += "<b>ZEMĚPISNÁ DÉLKA</b> není desetinné číslo nebo nemá tvar např. 14:21:32.445E";
						countErr++;
					}
				}
				latitude = rs.getString("LATITUDEORIG");
				if(StringUtils.isBlank(latitude)){
					result += addDelim(result);
					result += "chybí <b>ZEMĚPISNÁ ŠÍŘKA</b>";
					countErr++;
				}else{
					if(utils.transformLatitude(latitude) == null){
						result += addDelim(result);
						result += "<b>ZEMĚPISNÁ ŠÍŘKA</b> není desetinné číslo nebo nemá tvar STUPNĚ:MINUTY:VTEŘINY";
						countErr++;
					}
				}
				namedplace = rs.getString("NAMEDPLACE");
				if(StringUtils.isBlank(namedplace)){
					if(utils.transformLatitude(latitude) != null && utils.transformLongitude(longitude) != null){
						String code = rm.getCountry(Double.parseDouble(latitude), Double.parseDouble(longitude));
						iim.setCountryCode(id, code);
					}else{
					}
				}
				countryCode = rs.getString("COUNTRY");
				if(StringUtils.isBlank(countryCode) == false){
					country = rm.isCountryCode(countryCode);
					if(country.get(0)[0].equals("##")){
						result += addDelim(result);
						result += "<b>" +countryCode + "</b> není dvoupísmenný <b>KÓD STÁTU</b> dle ISO 3166-1";
					}
				}
				coordaccu = rs.getString("COORDACCURACY");
				if(StringUtils.isBlank(coordaccu)){
					result += addDelim(result);
					result += "chybí <b>PŘESNOST SOUŘADNIC</b>";
					countErr++;
				}
				coordsrc = rs.getString("COORDSOURCE");
				if(StringUtils.isBlank(coordsrc)){
					result += addDelim(result);
					result += "chybí <b>ZDROJ SOUŘADNIC</b>";
				}
				altitude = rs.getString("ALTITUDE");
				if(StringUtils.isBlank(altitude) == false){
					if(utils.isInteger(altitude) == false){
						result += addDelim(result);
						result += "<b>NADMOŘSKÁ VÝŠKA</b> není celé číslo";
						countErr++;
					}
				}
				biotope = rs.getString("BIOTOPE");
				if(StringUtils.isBlank(biotope) == false){
					if(biotope.length() > 100){
						result += addDelim(result);
						result += "<b>BIOTOP</b> je delší než 100 znaků";
						countErr++;
					}
				}
				abund = rs.getString("ABUND");
				if(StringUtils.isBlank(abund) == false){
					if(abund.length() > 100){
						result += addDelim(result);
						result += "<b>POČETNOST</b> je delší než 100 znaků";
						countErr++;
					}
				}
				locnote = rs.getString("LOCALITYNOTE");
				if(StringUtils.isBlank(locnote) == false){
					if(locnote.length() > 255){
						result += addDelim(result);
						result += "<b>POZNÁMKA K LOKALITĚ</b> je delší než 255 znaků";
						countErr++;
					}
				}
				occnote = rs.getString("OCCURENCENOTE");
				if(StringUtils.isBlank(occnote) == false){
					if(occnote.length() > 255){
						result += addDelim(result);
						result += "<b>POZNÁMKA K NÁLEZU</b> je delší než 255 znaků";
						countErr++;
					}
				}
				date1 = rs.getString("DATE1ORIG");
				if(StringUtils.isBlank(date1)){
					result += addDelim(result);
					result += "Chybí zapsané <b>DATUM 1</b>";
					countErr++;
				}else{
					if(StringUtils.isBlank(utils.transformDate(date1))){
						result += addDelim(result);
						result += "<b>DATUM 1</b> není ve formátu DD. MM. RRRR, DD.MM.RRRR, RRRR-MM-DD, RRRR-MM, RRRR nebo s.d.";
						countErr++;
					}
				}
				date2 = rs.getString("DATE2");
				if(StringUtils.isBlank(date2) == false){
					if(StringUtils.isBlank(utils.transformDate(date2))){
						result += addDelim(result);
						result += "<b>DATUM 2</b> není ve formátu DD. MM. RRRR, DD.MM.RRRR, RRRR-MM-DD, RRRR-MM, RRRR nebo s.d.";
						countErr++;
					}
				}
				collectors = rs.getString("COLLECTORS");
				if(StringUtils.isBlank(collectors)){
					result += addDelim(result);
					result += "chybí zapsaní <b>NÁLEZCI</b>";
					countErr++;
				}
				herb = rs.getString("HERBARIUM");
				if(StringUtils.isBlank(herb) == false){
					if(herb.length() > 255){
						result += addDelim(result);
						result += "<b>HERBÁŘ</b> je delší než 255 znaků";
						countErr++;
					}
				}
				source = rs.getString("SOURCE");
				if(StringUtils.isBlank(source)){
					result += addDelim(result);
					result += "chybí zapsaný <b>ZDROJ</b>";
					countErr++;
				}
				project = rs.getString("PROJECT");
				if(StringUtils.isBlank(project)){
					result += addDelim(result);
					result += "chybí zapsaný <b>PROJEKT</b>";
					countErr++;
				}
				negative = rs.getString("NEGATIVE");
				if(StringUtils.isBlank(negative)==false){
					if(utils.isNegative(negative) == false){
						result += addDelim(result);
						result += "<b>NEGATIVNÍ NÁLEZ</b> není celé číslo v rozmezí 0-1 nebo logická hodnota true/false";
						countErr++;
					}
				}
				det = rs.getString("DETERMINATED");
				if(StringUtils.isBlank(det) == false){
					if(det.length() > 100){
						result += addDelim(result);
						result += "<b>DETERMINOVAL</b> je delší než 100 znaků";
						countErr++;
					}
				}
				rev = rs.getString("REVISED");
				if(StringUtils.isBlank(rev) == false){
					if(rev.length() > 100){
						result += addDelim(result);
						result += "<b>REVIDOVAL</b> je delší než 100 znaků";
						countErr++;
					}
				}
				schedacount = rs.getString("SCHEDACOUNT");
				if(StringUtils.isBlank(schedacount) == false){
					if(utils.isInteger(schedacount) == false){
						result += addDelim(result);
						result += "<b>POČET SCHED/HERBÁŘOVÝCH LISTŮ</b> není celé číslo";
						countErr++;
					}
				}
				countSumErr += countErr;
				if(StringUtils.isBlank(result)== false){
					countErrRow++;
					result2 += "<p style='color:red'>Nalezeno <span style ='font-weight:bold'>"+ String.valueOf(countErr) +"</span> chyb na řádku č. <span style ='font-weight:bold'>" + String.valueOf(id) + "</span></p>";
					result2 += "<p>";
					result2 += "<span style='font-weight:bold'>Řádek <span style='color:red'>"+ String.valueOf(id) + "</span></span>, " + result + "<br />";
					result2 += "</p>";
				}
				result = "";
				rs.next();
				countErr = 0;
			}
			String duplicity = getDuplicity();
			
			if(countErrRow == 0){
				result2 += "<span style = 'color:green; font-weight:bold'>Nebyly nalezeny žádné chyby</span>";
				result2 += duplicity;
				separate_data.setDisable(false);
				if(duplicity.length() > 0){
					delete_duplicity.setDisable(false);
				}else{
					delete_duplicity.setDisable(true);
				}
				table_input.setEditable(false);
				setComputedValue();
			}else{
				result2 = "<p style='font-family:sans-serif;color:red;font-size:14px'>Nalezeno celkem <span style ='font-weight:bold'>"+ String.valueOf(countSumErr) +"</span> chyb na <span style ='font-weight:bold'>" + countErrRow + "</span> řádcích</p>" + result2;
				result2 += duplicity;
				separate_data.setDisable(true);
				if(duplicity.length() > 0){
					delete_duplicity.setDisable(false);
				}else{
					delete_duplicity.setDisable(true);
				}
			}
			result2 += "</div>";

		}catch(Exception e){
			String descr = "Nezdařilo se zkontrolovat data z tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return result2;
	}
	
	/**
	 * 
	 */
	private void setComputedValue(){
		ResultSet rs = iim.getUpdatableData();
		try{
			while(rs.next()){
				rs.updateDouble("longitude", utils.transformLongitude(rs.getString("longitudeorig")));
				rs.updateDouble("latitude", utils.transformLatitude(rs.getString("latitudeorig")));
				rs.updateString("date1", utils.transformDate(rs.getString("date1orig")));
				if(rs.getString("date2orig") != null){
					rs.updateString("date2", utils.transformDate(rs.getString("date2orig")));
				}
				
				rs.updateRow();
			}
		}catch(SQLException e){
			e.printStackTrace();
			String descr = "Nezdařilo se transformovat souřadnice a datumy";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}

	private void showCheckResult(String result){
		check_result.getEngine().loadContent(utils.getHtmlHeader() + result +utils.getHtmlFoot());
		check_result.getEngine().reload();
	}

	/**
	 * Add semicolon to string if it is not null or empty
	 * @param result 
	 * @return String
	 */
	private String addDelim(String s){
		String result = "";
		if (StringUtils.isBlank(s) == false){
			result += "; ";
		}
		return result;
	}

	@FXML
	public void handleSeparateData(){
		Task task = new Task() {
			@Override
			protected Object call() throws Exception {
				Stage stage = (Stage)separate_data.getScene().getWindow();
				stage.getScene().setCursor(Cursor.WAIT);
				tm.createTable();
				tm.importData();
				tc.handleReload();
		
				cm.createTable();
				cm.importData();
				cc.handleReload();
		
				pm.createTable();
				pm.importData();
				pc.handleReload();
		
				sm.createTable();
				sm.importData();
				sc.handleReload();
		
				npm.createTable();
				npm.importData();
				npc.handleReload();
		
				phm.createTable();
				phm.importData();
				phc.handleReload();
		
				cam.createTable();
				cam.importData();
				cac.handleReload();
		
				csm.createTable();
				csm.importData();
				csc.handleReload();
		
				separate_data.setDisable(true);
				
				stage.getScene().setCursor(Cursor.DEFAULT);
				return null;
			}
		};
		task.run();
	}
	
	public void setTableEditable(){
		table_input.setEditable(true);
	}
	
	private String getDuplicity() throws SQLException{
		ResultSet rs = iim.getDuplicity();
		rs.last();
		int count = rs.getRow();
		String result = "";
		if(count > 0){
			rs.first();
			result = "<div>";
			result += "<p style='color:black;font-size:14px'>Celkový počet duplicitních záznamů: <span style='font-weight:bold'>" + count +"</span></p>";
			for(int i = 0; i<count;i++){
				result += "<p>";
				result += "<span style='font-weight:bold'> Počet duplicit:</span> &nbsp;" + rs.getInt(1) + "<br />";
				result += "<span style='font-weight:bold'> Taxon:</span> &nbsp;" + rs.getString("taxon") + "<br />";
				result += "<span style='font-weight:bold'> Lokalita:</span> &nbsp;" + rs.getString("locality") + "<br />";
				result += "<span style='font-weight:bold'> Zeměpisná šířka:</span> &nbsp;" + rs.getString("latitudeorig") + "<br />";
				result += "<span style='font-weight:bold'> Zeměpisná délka:</span> &nbsp;" + rs.getString("longitudeorig") + "<br />";
				result += "<span style='font-weight:bold'> Poznámka k lokalitě:</span> &nbsp;" + rs.getString("localitynote") + "<br />";
				result += "<span style='font-weight:bold'> Nálezci:</span> &nbsp;" + rs.getString("collectors") + "<br />";
				result += "<span style='font-weight:bold'> Datum 1:</span> &nbsp;" + rs.getString("date1orig") + "<br />";
				result += "<span style='font-weight:bold'> Datum 2:</span> &nbsp;" + rs.getString("date2orig") + "<br />";
				result += "<span style='font-weight:bold'> Poznámka k nálezu:</span> &nbsp;" + rs.getString("occurencenote") + "<br />";
				result += "<span style='font-weight:bold'> Poznámka na schedě:</span> &nbsp;" + rs.getString("schedanote") + "<br />";
				result += "</p>";
				rs.next();
			}
			result += "</div>";
		}
		return result;
	}
}
