package org.hansv.vratickaimport3;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.concurrent.ExecutionException;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import org.hansv.db.H2db;
import org.hansv.model.CoordaccuracyModel;
import org.hansv.model.CoordsourceModel;
import org.hansv.model.RestItem;
import org.hansv.model.InputImportModel;
import org.hansv.model.LocalityModel;
import org.hansv.model.LocalityOccurencesItem;
import org.hansv.model.NamedPlaceModel;
import org.hansv.model.OccurenceItem;
import org.hansv.model.PhytochorionModel;
import org.hansv.model.RestModel;
import org.hansv.utils.LoginSingleton;
import org.hansv.utils.Utils;


/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class LocalityOccurencesImport extends Service<String>{
	private LocalityModel lm;
	private PhytochorionModel pm;
	private NamedPlaceModel npm;
	private CoordaccuracyModel cam;
	private CoordsourceModel csm;
	private RestModel rm;
	private LoginSingleton ls;
	private InputImportModel iim;
	private Utils utils;
	private H2db db;
	
	public LocalityOccurencesImport(){
		ls = LoginSingleton.getInstance();
	}
	
	/**
	 * Setter injector for class LocalityModel
	 * @param lm Class LocalityModel
	 * @see org.hansv.model.LocalityModel
	 */
	public void setLocalityModel(LocalityModel lm){
		this.lm = lm;
	}
	
	/**
	 * Setter injector for class RestModel
	 * @param rm Class RestModel
	 * @see org.hansv.model.RestModel
	 */
	public void setRestModel(RestModel rm){
		this.rm = rm;
	}
	
	/**
	 * Setter injector for class PhytochorionModel
	 * @param pm Class PhytochorionModel
	 * @see org.hansv.model.PhytochorionModel
	 */
	public void setPhytochorionModel(PhytochorionModel pm){
		this.pm = pm;
	}
	
	/**
	 * Setter injector for class NamedPlaceModel
	 * @param npm Class NamedPlaceModel
	 * @see org.hansv.model.NamedPlaceModel
	 */
	public void setNamedplaceModel(NamedPlaceModel npm){
		this.npm = npm;
	}
	
	/**
	 * Setter injector for class CoordaccuracyModel
	 * @param cam Class CoordaccuracyModel
	 * @see org.hansv.model.CoordaccuracyModel
	 */
	public void setCoordaccuracyModel(CoordaccuracyModel cam){
		this.cam = cam;
	}
	
	/**
	 * Setter injector for class CoordsourceModel
	 * @param csm Class CoordsourceModel
	 * @see org.hansv.model.CoordsourceModel
	 */
	public void setCoordsourceModel(CoordsourceModel csm){
		this.csm = csm;
	}
	
	/**
	 * Setter injector for class InputImportModel
	 * @param iim Class CoordsourceModel
	 * @see org.hansv.model.InputImportModel
	 */
	public void setInputImportModel(InputImportModel iim){
		this.iim = iim;
	}
	
	/**
	 * Setter injector for class Utils
	 * @param utils Class Utils
	 * @see org.hansv.utils.Utils
	 */
	public void setUtils(Utils utils){
		this.utils = utils;
	}
	
	/**
	 * Setter injector for class H2db
	 * @param h2 Class H2db
	 * @see org.hansv.db.H2db
	 */
	public void setH2db(H2db h2){
		db  = h2;
	}
	
	/**
	 * Thread for saving locality
	 * @return 
	 */
	@Override
	protected Task<String> createTask() {
		return new Task<String>(){
			@Override
			protected String call()throws Exception{
				updateTitle("Získávání dat");
				ResultSet rs = lm.getLocality();
				updateTitle("Import dat");
				RestItem ri;
				if(rs != null){
					int i = 0;
					rs.last();
					int size = rs.getRow();
					rs.beforeFirst();
					int importid = 0;
					while(rs.next()){
						updateProgress(i + 1, size);
						if (isCancelled() == true) {
							return null;
						}
						importid = rs.getInt("importid");
						OccurenceItem[] oi = iim.getOccurences4locality(rs.getString("locality"), rs.getDouble("longitude"), rs.getDouble("latitude"), rs.getString("namedplaceorig"), rs.getString("area"), rs.getString("phytochorionorig"), rs.getString("kvadrant"), rs.getString("coordaccuracyorig"), rs.getString("coordsourceorig"), rs.getString("altitude"), rs.getString("note"), rs.getString("country"));
						if (isCancelled() == true) {
							return null;
						}
						
						ri = rm.saveLocalityOccurences(new LocalityOccurencesItem(rs.getString("locality"), num2string(rs.getDouble("latitude")), num2string(rs.getDouble("longitude")), num2string(rs.getInt("altitude")), csm.findId((rs.getString("coordsourceorig"))), cam.findId(rs.getString("coordaccuracyorig")), getPhytochorion(rs.getString("phytochorionorig")), npm.findId(rs.getString("namedplaceorig")),rs.getString("country") ,rs.getString("area"), rs.getString("kvadrant"), rs.getString("note"),Integer.toString(rs.getInt("importid")) ,ls.getName(), ls.getPassword(),oi));
						if (isCancelled() == true) {
							return null;
						}
						if(ri.getMap().get("id") == null){
							String msg = "<div style='font-family:sans-serif;font-size:12px;color:red;font-weight: bold;'>" + utils.decodeHttpCode(ri.getHttpcode()) + "</div>";
							msg += "<div style='font-family:sans-serif;font-size:12px;color:black'>";
							msg += "<span style='font-weight: bold;'>Kod chyby:</span>&nbsp;" + ri.getHttpcode() + "<br />";
							msg += "<span style='font-weight: bold;'>Popis chyby:</span>&nbsp;" + utils.decodeHttpCode(ri.getHttpcode()) + "<br />";
							msg += "<span style='font-weight: bold;'>Lokalita:</span>&nbsp;" + rs.getString("locality") + "<br />";
							msg += "<span style='font-weight: bold;'>Zeměpisná šířka:</span>&nbsp;" + num2string(rs.getDouble("latitude")) + "<br />";
							msg += "<span style='font-weight: bold;'>Zeměpisná délka:</span>&nbsp;" + num2string(rs.getDouble("longitude")) + "<br />";
							msg += "<span style='font-weight: bold;'>Nadmořská výška:</span>&nbsp;" + num2string(rs.getDouble("altitude")) + "<br />";
							msg += "<span style='font-weight: bold;'>Zdroj souřadnic:</span>&nbsp;" + csm.findId((rs.getString("coordsourceorig"))) + "<br />";
							msg += "<span style='font-weight: bold;'>Přesnost souřadnic:</span>&nbsp;" + cam.findId(rs.getString("coordaccuracyorig")) + "<br />";
							msg += "<span style='font-weight: bold;'>Kód fytochorionu:</span>&nbsp;" + getPhytochorion(rs.getString("phytochorionorig")) + "<br />";
							msg += "<span style='font-weight: bold;'>Kód nejbližšího sídla:</span>&nbsp;" + npm.findId(rs.getString("namedplaceorig")) + "<br />";
							msg += "<span style='font-weight: bold;'>Oblast:</span>&nbsp;" + rs.getString("area") + "<br />";
							msg += "<span style='font-weight: bold;'>Pole sítového mapování:</span>&nbsp;" + rs.getString("kvadrant") + "<br />";
							msg += "<span style='font-weight: bold;'>Poznámka:</span>&nbsp;" + rs.getString("note") + "<br />";
							for(i=0; i<oi.length; i++){
								msg += "<span style='font-weight: bold;'>Taxon:</span>&nbsp;" + oi[i].taxon + "<br />";
								msg += "<span style='font-weight: bold;'>Originální jméno taxonu:</span>&nbsp;" + oi[i].originalname + "<br />";
								msg += "<span style='font-weight: bold;'>Kvalifikátor určení:</span>&nbsp;" + oi[i].abbrev + "<br />";
								msg += "<span style='font-weight: bold;'>Datum 1:</span>&nbsp;" + oi[i].date1 + "<br />";
								msg += "<span style='font-weight: bold;'>Datum 2:</span>&nbsp;" + oi[i].date2 + "<br />";
								msg += "<span style='font-weight: bold;'>Negativní:</span>&nbsp;" + oi[i].negative + "<br />";
								msg += "<span style='font-weight: bold;'>Biotop:</span>&nbsp;" + oi[i].biotope + "<br />";
								msg += "<span style='font-weight: bold;'>Nálezci:</span>&nbsp;" + oi[i].collectors + "<br />";
								msg += "<span style='font-weight: bold;'>Početnost:</span>&nbsp;" + oi[i].abund + "<br />";
								msg += "<span style='font-weight: bold;'>Herbář:</span>&nbsp;" + oi[i].herbarium + "<br />";
								msg += "<span style='font-weight: bold;'>Poznámka:</span>&nbsp;" + oi[i].note + "<br />";
								msg += "<span style='font-weight: bold;'>Projekt:</span>&nbsp;" + oi[i].project + "<br />";
								msg += "<span style='font-weight: bold;'>Zdroj:</span>&nbsp;" + oi[i].source + "<br />";
								msg += "<span style='font-weight: bold;'>Status:</span>&nbsp;" + oi[i].status + "<br />";
								msg += "<span style='font-weight: bold;'>Determinoval:</span>&nbsp;" + oi[i].authordet + "<br />";
								msg += "<span style='font-weight: bold;'>Revidoval:</span>&nbsp;" + oi[i].authorrev + "<br />";
							}
							msg += "</div>";
							return msg;
						}else{
							lm.setLocalityId(ri.getMap().get("id"), rs.getInt("id"));
							if (isCancelled() == true) {
							return null;
						}
							iim.setLocalityIdOcc(ri.getMap().get("id"), rs.getString("locality"), rs.getDouble("longitude"), rs.getDouble("latitude"), rs.getString("namedplaceorig"), rs.getString("area"), rs.getString("phytochorionorig"), rs.getString("kvadrant"), rs.getString("coordaccuracyorig"), rs.getString("coordsourceorig"), rs.getString("altitude"), rs.getString("note"));
						}
						updateProgress(i + 1, size);
						i++;
					}
					RestItem count = rm.getCountImported(importid);
					if(Integer.parseInt(count.getMap().get("locality")) != lm.getCount() ||Integer.parseInt(count.getMap().get("occurence")) != iim.getCountData()){
						updateMessage("Počet originálních lokalit " + lm.getCount() + " počet importovaných lokalit " + count.getMap().get("locality") + "<br />počet originálních nálezů " + iim.getCountData() + " počet importovaných nálezů " + count.getMap().get("occurence"));
						throw new ExecutionException(new Throwable("Count is bad"));
					}
				}
				return "Completed";				
			}
		};
	}
	
	private String num2string(Double value){
		return value.toString();
	}
	
	private String num2string (Integer value){
		return value.toString();
	}
	
	private String getPhytochorion(String phytochorion) throws SQLException{
		String id = null;
		if(phytochorion != null){
			if(phytochorion.length() > 0){
				id = pm.findId(phytochorion);
			}
		}
		return id;
	}
}
