package org.hansv.vratickaimport3;


import com.google.inject.Guice;
import com.google.inject.Injector;
import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.fxml.JavaFXBuilderFactory;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.control.PasswordField;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javafx.scene.input.KeyCode;
import javafx.stage.Stage;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.LoginItem;
import org.hansv.model.RestModel;
import org.hansv.utils.DepInjectionModule;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.FXMLControllersLoader;
import org.hansv.utils.LoginSingleton;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class LoginController implements Initializable {
	@FXML
	private TextField name;
	@FXML
	private PasswordField password;
	@FXML
	private Button login;
	@FXML
	private Button proxysettings;
	
	private RestModel rm;
	private Utils utils;
	private Messages msg;
	private ErrorLog errlog;

	
	/**
	 * Initializes the controller class.
	 * @param url url
	 * @param rb ResourceBundle
	 */
	@Override
	public void initialize(URL url, ResourceBundle rb) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		rm = (RestModel)context.getBean("restModel");
		utils = (Utils)context.getBean("utils");
		errlog = (ErrorLog)context.getBean("errorLog");
		utils.setUp();
		msg = (Messages)context.getBean("messages");
		name.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>() {
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if (t.getCode() == KeyCode.ENTER) {
					handleLogin();
				}
			}
		});
		password.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>() {
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if (t.getCode() == KeyCode.ENTER) {
					handleLogin();
				}
			}
		});
		login.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>() {
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if (t.getCode() == KeyCode.ENTER) {
					handleLogin();
				}
			}
		});
		proxysettings.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>() {
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if (t.getCode() == KeyCode.ENTER) {
					handleProxy();
				}
			}
		});
		Tooltip tp = new Tooltip();
		tp.setText("Přihlásit");
		login.setTooltip(tp);
		Tooltip tp1 = new Tooltip();
		tp1.setText("Nastavit proxy server");
		proxysettings.setTooltip(tp1);
		Platform.runLater(()->name.requestFocus());
	}

	@FXML
	public void handleLogin(){
		try{
			LoginItem li = rm.login(name.getText(), password.getText());
			if(li.getStatus() == true){
				Stage stageLogin = (Stage) login.getScene().getWindow();
				stageLogin.close();
				final Injector injector = Guice.createInjector(new DepInjectionModule());
				final FXMLLoader loader = new FXMLControllersLoader(getClass().getResource("/fxml/Scene.fxml"), null, new JavaFXBuilderFactory(), injector);
				final Parent view = loader.load();
				Scene scene = new Scene(view);
				Stage stageData = new Stage();
				String theme = rm.getTheme();
				String url = utils.getServerPath() + "css/"+ theme +".css";
				scene.getStylesheets().add("/styles/Styles.css");
				scene.getStylesheets().add(url);
				stageData.setTitle("Vratička import");
				stageData.getIcons().add(new Image("/images/favicon.png"));
				stageData.setScene(scene);
				LoginSingleton ls = LoginSingleton.getInstance();
				ls.setName(name.getText());
				ls.setPassword(password.getText());
				stageData.show();
			}else{
				msg.showWarn(String.valueOf(li.getStatus()),li.getMessage()); 
			}
		}catch(Exception e){
			msg.showErr("Chyba získání url", ExceptionUtils.getStackTrace(e));
			e.printStackTrace();
		}
	}
	
	@FXML
	public void handleProxy(){
		try{
			final Injector injector = Guice.createInjector(new DepInjectionModule());
			final FXMLLoader loader = new FXMLControllersLoader(getClass().getResource("/fxml/Proxysetting.fxml"), null, new JavaFXBuilderFactory(), injector);
			final Parent view = loader.load();
			Stage stageLogin = (Stage) login.getScene().getWindow();
			stageLogin.close();
			Scene scene = new Scene(view);
			Stage stageData = new Stage();
			String theme = rm.getTheme();
			String url = utils.getServerPath() + "css/"+ theme +".css";
			scene.getStylesheets().add("/styles/Styles.css");
			scene.getStylesheets().add(url);
			stageData.setTitle("Vratička import");
			stageData.setScene(scene);
			stageData.show();
		}catch(IOException e){
			String descr = "Chyba programu";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
}
