package org.hansv.vratickaimport3;

import java.io.IOException;
import java.util.HashMap;
import javafx.application.Application;
import static javafx.application.Application.launch;
import javafx.application.Platform;
import javafx.event.EventHandler;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.stage.Stage;
import javafx.stage.WindowEvent;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.ProxyModel;
import org.hansv.model.RestModel;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.ProxySingleton;
import org.hansv.utils.SettingSingleton;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;


/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class MainApp extends Application {

	private ErrorLog errlog;
	ApplicationContext context;
	private Messages msg;
	
	
	
	@Override
	public void start(final Stage stage) throws Exception {
		context = new ClassPathXmlApplicationContext("config/Beans.xml");
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		Utils utils = (Utils)context.getBean("utils");
		utils.setUp();
		setProxy();
		RestModel rest = (RestModel)context.getBean("restModel");
		String theme = rest.getTheme();
		String url = utils.getServerPath() + "css/"+ theme +".css";
		SettingSingleton setSin = SettingSingleton.getInstance();
		setSin.setUrlTheme(url);
		stage.getIcons().add(new Image("/images/favicon.png"));
		stage.setOnCloseRequest(new EventHandler<WindowEvent>(){
			@Override
			public void handle(WindowEvent e){
				Platform.exit();
				System.exit(0);
			}
		});
		if(theme != null){
			showLogin(stage, url);
		}else{
			showProxyConfig(stage);
			msg.showWarn( "Nelze s připojit k serveru " + utils.getServerPath() + ". Zkuste nastavit proxy server","Chyba připojení");
		}
	}
	
	private void setProxy(){
		ProxyModel pm = (ProxyModel)context.getBean("proxyModel");
		HashMap<String, String> proxyValue = pm.getConfig();
		ProxySingleton ps = ProxySingleton.getInstance();
		if(proxyValue.get("host") != null){
			if(proxyValue.get("host").length() > 0){
				ps.setHost(proxyValue.get("host"));
			}else{
				ps.setHost(null);
			}
		}else{
			ps.setHost(null);
		}
	
			
		if(proxyValue.get("port") != null){
			if(proxyValue.get("port").length() > 0){
				ps.setPort(Integer.parseInt(proxyValue.get("port")));
			}else{
				ps.setPort(-1);
			}
		}else{
			ps.setPort(-1);
		}
		if(proxyValue.get("user") != null){
			if(proxyValue.get("user").length() > 0){
				ps.setUser(proxyValue.get("user"));
			}else{
				ps.setUser(null);
			}
		}else{
			ps.setUser(null);
		}
		if(proxyValue.get("password") != null){
			if(proxyValue.get("password").length() > 0){
				ps.setPassword(proxyValue.get("password"));
			}else{
				ps.setPassword(null);
			}
		}else{
			ps.setPassword(null);
		}
	}
	
	private void showLogin(Stage stage, String url){
		try {
			Parent root = FXMLLoader.load(getClass().getResource("/fxml/Login.fxml"));
			Scene scene = new Scene(root);
			scene.getStylesheets().add("/styles/Styles.css");
			scene.getStylesheets().add(url);
			stage.setTitle("Vratička import");
			stage.setScene(scene);
			stage.show();
		} catch (IOException e) {
			errlog.logErr(ExceptionUtils.getStackTrace(e), "Main.showLogin");
		}
	}
	
	private void showProxyConfig(Stage stage){
		try {
			Parent root = FXMLLoader.load(getClass().getResource("/fxml/Proxysetting.fxml"));
			Scene scene = new Scene(root);
			scene.getStylesheets().add("/styles/Styles.css");
			stage.setTitle("Vratička import");
			stage.setScene(scene);
			stage.show();
		} catch (IOException e) {
			errlog.logErr(ExceptionUtils.getStackTrace(e), "Main.showProxyConfig");
		}
	}

	/**
	 * @param args the command line arguments
	 */
	public static void main(String[] args) {
		launch(args);
	}
}
