package org.hansv.vratickaimport3;


import com.google.inject.Singleton;
import java.io.IOException;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.beans.property.ReadOnlyObjectProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Task;
import javafx.concurrent.Worker;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Cursor;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TablePosition;
import javafx.scene.control.Tooltip;
import javafx.scene.control.cell.TextFieldTableCell;
import javafx.scene.image.Image;
import javafx.scene.input.KeyCode;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.scene.web.WebView;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.util.Callback;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.InputImportModel;
import org.hansv.model.NamedPlaceItem;
import org.hansv.model.NamedPlaceModel;
import org.hansv.model.RestModel;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class NamedPlaceController extends BaseController{

	@FXML
	private ProgressIndicator indicator_map_place;
	@FXML
	private Button reload_data;
	@FXML
	private Button map_places;
	@FXML
	private Button clear_places;
	@FXML
	private Button save_place;
	@FXML
	private Button delete_place;
	@FXML
	private Button add_place;
	@FXML
	private Button decomp_place;
	@FXML
	private WebView www_analyze;
	
	private NamedPlaceModel npm;
	private RestModel rm;
	private InputImportModel iim;


	
	@Override
	public void initialize(URL url, ResourceBundle rb) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		npm = (NamedPlaceModel)context.getBean("namedPlaceModel");
		model = (NamedPlaceModel)context.getBean("namedPlaceModel");
		utils = (Utils)context.getBean("utils");
		du = (DbUtils)context.getBean("dbUtils");
		rm = (RestModel)context.getBean("restModel");
		iim = (InputImportModel)context.getBean("inputImportModel");
		if(du.tableExist("NAMEDPLACE") == false){
			reload_data.setDisable(true);
		}
		combo_value.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>(){
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if(combo_value.getEditor().getText().length() > 1 && t.getCode().isArrowKey() == false && t.getCode() != KeyCode.ENTER){
					comboThread(combo_value.getEditor().getText());
				}
			}
		});
		combo_value.getEditor().focusedProperty().addListener(
			(observable, oldValue, newValue) -> {
				if (newValue) {
					Platform.runLater(combo_value.getEditor()::selectAll);
				}
			}
		);
		table_data.setOnMouseClicked(new EventHandler<MouseEvent>() {
			@Override
			public void handle(MouseEvent mouseEvent) {
				if(mouseEvent.getButton().equals(MouseButton.PRIMARY)){
					if(mouseEvent.getClickCount() == 2){
						ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
						TablePosition tablePosition = (TablePosition) selectedCells.get(0);
						ObservableList row = data.get(tablePosition.getRow());
						getStatistic(row.get(1).toString());
					}
				}
			}
		});
		Tooltip tp1 = new Tooltip();
		tp1.setText("Zobrazení dat");
		reload_data.setTooltip(tp1);
		Tooltip tp2 = new Tooltip();
		tp2.setText("Zrušení přiřazení sídel");
		clear_places.setTooltip(tp2);
		Tooltip tp3 = new Tooltip();
		tp3.setText("Namapování sídel");
		map_places.setTooltip(tp3);
		Tooltip tp4 = new Tooltip();
		tp4.setText("Přiřazení sídla");
		save_place.setTooltip(tp4);
		Tooltip tp5 = new Tooltip();
		tp5.setText("Smazat přiřazení sídla");
		delete_place.setTooltip(tp5);
		Tooltip tp6 = new Tooltip();
		tp6.setText("Nové sídlo");
		add_place.setTooltip(tp6);
		Tooltip tp7 = new Tooltip();
		tp7.setText("Lokality k sídlu");
		decomp_place.setTooltip(tp7);
		showResult("");
		handleReload();
	}
	
	@FXML
	public void handleReload(){
		if(du.tableExist("NAMEDPLACE")){
			setTableColumns();
			loadData();
		}
	}
	
	@FXML
	public void handleMap(){
		www_analyze.getEngine().loadContent(utils.getHtmlHeader() + utils.getHtmlFoot());
		www_analyze.getEngine().reload();
		npm.setOnFailed((event)-> {
			Throwable t = npm.getException();
			String descr = "Chyba mapování sídel";
			errlog.logErr(ExceptionUtils.getStackTrace(t), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(t);
			msg.showErr(errs[0] + errs[1], descr);
		});
		indicator_map_place.progressProperty().bind(npm.progressProperty());
		npm.reset();
		npm.setParams(data, rm);
		npm.start();
		final ReadOnlyObjectProperty<Worker.State> state = npm.stateProperty();
		state.addListener(new ChangeListener<Worker.State>() {
			@Override
			public void changed(ObservableValue<? extends Worker.State> observable, Worker.State oldValue, Worker.State newValue) {
				if(newValue.toString().equals("SUCCEEDED")){
					if(npm.getValue().equals("Completed")){
						getStatistic(null);
					}
				}
			}
		});
	}
	
	@FXML
	public void handleClear(){
		clearData();
	}
	
	@FXML
	public void handleSave(){
		save();
	}
	
	@FXML
	public void handleDelete(){
		delete();
	}
	
	@FXML
	public void handleAdd(ActionEvent event){
		try {
			FXMLLoader loader = new FXMLLoader(getClass().getResource("/fxml/WWWBrowser.fxml"));
			ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
			Stage stage = new Stage(StageStyle.DECORATED);
			stage.setScene(new Scene((Pane) loader.load()));
			WWWBrowserController controller = loader.<WWWBrowserController>getController();
			if(selectedCells.isEmpty()){
				controller.load("namedplace/editremote");
			}else{
				TablePosition tablePosition = (TablePosition) selectedCells.get(0);
				ObservableList row = data.get(tablePosition.getRow());
				Double[] coords = iim.getCoordslocalityByPlace(row.get(1).toString());
				controller.loadNamedPlace("namedplace/editremote",coords[0],coords[1]);
			}
			stage.show();
			
		} catch (IOException e) {
			String descr = "Nezdařilo se otevřít okno s vestavěným WWW prohlížečem";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	@FXML
	public void handleDecompPlace(ActionEvent event){
		ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
		if(selectedCells.isEmpty()){
			msg.showWarn("Nemáte vybrané sídlo, které chcete analyzovat.", "Vyberte ho v tabulce");
			return;
		}

		TablePosition tablePosition = (TablePosition) selectedCells.get(0);
		ObservableList row = data.get(tablePosition.getRow());
		openDecomp(row.get(1).toString());
	}
	
	
	private void openDecomp(String place){
		try{
			ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
			FXMLLoader loader = new FXMLLoader(getClass().getResource("/fxml/DecompPlace.fxml"));
			Stage stage = new Stage(StageStyle.DECORATED);
			rm = (RestModel)context.getBean("restModel");
			String theme = rm.getTheme();
			utils = (Utils)context.getBean("utils");
			String url = utils.getServerPath() + "css/"+ theme +".css";
			Scene scene = new Scene((Pane) loader.load());
			scene.getStylesheets().add("/styles/Styles.css");
			scene.getStylesheets().add(url);
			stage.setScene(scene);
			stage.setTitle("Vratička import");
			stage.getIcons().add(new Image("/images/favicon.png"));
			DecompPlaceController controller = loader.<DecompPlaceController>getController();
			controller.initData(place);
			controller.setNamedPlaceController(this);
			stage.show();
		} catch (IOException e) {
			String descr = "Nezdařilo se otevřít okno s analýzou sídla";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	

	@Override
	public void save(){
		Object nameObj = combo_value.getValue();
		if(nameObj == null){
			msg.showWarn("Nemáte zadanou hodnotu, kterou chcete přiřadit", "Zadejte ji ve výběrovém seznamu");
			return;
		}
		ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
		if(selectedCells.isEmpty()){
			msg.showWarn("Nemáte vybraný řádek, ke kterému chcete přiřadit hodnotu.", "Vyberte ho v tabulce");
			return;
		}
		if(utils.getKey(nameObj.toString(),comboData) == null){
			msg.showWarn("Zapsaná hodnota ve výběrovém seznamu není platná", "Vyberte jí znova");
			return;
		}
		TablePosition tablePosition = (TablePosition) selectedCells.get(0);
		ObservableList row = data.get(tablePosition.getRow());
		row.set(2, nameObj.toString());
		row.set(3, utils.getKey(nameObj.toString(),comboData));
		String[] coords2 = getCoords(nameObj.toString(), comboData);
		row.set(4, coords2[1]);
		row.set(5, coords2[0]);
		ResultSet dist = iim.getDistanceAnalyse(nameObj.toString());
		try{
			dist.first();
			row.set(6,Double.toString(dist.getDouble("max")));
		}catch(SQLException e){
			String descr = "Nezdařilo se načíst maximální vzdálenost";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		
		String[] values = new String[6];
		values[0] = row.get(1).toString();
		values[1] = nameObj.toString();
		values[2] = utils.getKey(nameObj.toString(),comboData);
		values[3] = coords2[0];
		values[4] = coords2[1];
		saveDistance(row.get(1).toString(), coords2[0], coords2[1]);
		ResultSet distance = iim.getDistanceAnalyse(row.get(1).toString());
		Double dblDist = null;
		try {
			distance.first();
			values[5] = Double.toString(distance.getDouble("max"));
			row.set(6, Double.toString(distance.getDouble("max")));
			dblDist = distance.getDouble("max");
		} catch (SQLException e) {
			String descr = "Nezdařilo se otevřít okno s analýzou sídla";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		data.set(tablePosition.getRow(), row);
		model.save(values);
	}
	
	@Override
	protected void clearData(){
		Thread t = new Thread(()->{
			Platform.runLater(new Runnable(){
				public void run(){
					Stage stage = (Stage)save_place.getScene().getWindow();
					stage.getScene().setCursor(Cursor.WAIT);
					String[] itemdata = new String[6];
					if(data == null){
						msg.showWarn("Není co mazat", "Načtěte data do tabulky tlačítkem \n\"Zobrazení dat\"");
						return;
					}
					for(int i = 0; i< data.size();i++){
						ObservableList row = data.get(i);
						itemdata[0] = row.get(1).toString();
						itemdata[1] = null;
						itemdata[2] = null;
						itemdata[3] = null;
						itemdata[4] = null;
						itemdata[5] = null;
						row.set(2, "");
						row.set(3, "");
						row.set(4, "");
						row.set(5,"");
						row.set(6, "");
						data.set(i, row);
						model.save(itemdata);
					}
					iim.saveNullDistance();
					stage.getScene().setCursor(Cursor.DEFAULT);
				}
			});
		});
		t.start();
		
	}
	
	@Override
	protected void delete(){
		ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
		if(selectedCells.isEmpty()){
			msg.showWarn("Nemáte vybrané, co chcete pročistit", "Vyberte v tabulce, co chcete smazat");
			return;
		}
		TablePosition tablePosition = (TablePosition) selectedCells.get(0);
		ObservableList row = data.get(tablePosition.getRow());
		row.set(2, "");
		row.set(3, "");
		row.set(4, "");
		row.set(5, "");
		row.set(6, "");
		data.set(tablePosition.getRow(), row);
		String[] rowdata = new String[6];
		rowdata[0] = row.get(1).toString();
		rowdata[1] = null;
		rowdata[2] = null;
		rowdata[3] = null;
		rowdata[4] = null;
		rowdata[5] = null;
		model.save(rowdata);
		iim.saveNullDistance(row.get(1).toString());
	}
	
	private void comboThread(String sample){
		Thread t = new Thread(()->{
			Platform.runLater(new Runnable(){
				public void run(){
					combo(rm.getNamedPlaces(sample));
				}
			});
		});
		t.start();
	}
	
	@Override
	protected void combo(ArrayList comboData2){
		String text = combo_value.getEditor().getText();
		comboData = comboData2;
		list = FXCollections.observableArrayList();
		for(int i = 0; i<comboData.size(); i++){
			String[] value = (String[])comboData.get(i);
			list.add(value[1]);
		}
		combo_value.setItems(list);
		combo_value.show();
		combo_value.getEditor().setText(text);
		combo_value.getEditor().end();
	}
	
	private void saveDistance(String place, String longitude, String latitude){
		Task task = new Task() {
			@Override
			protected Object call() throws Exception {
				Stage stage = (Stage)save_place.getScene().getWindow();
				stage.getScene().setCursor(Cursor.WAIT);
				iim.saveDistances(place, longitude, latitude);
				getStatistic(place);
				stage.getScene().setCursor(Cursor.DEFAULT);
				return null;
			}
		};
		task.run();
	}
	
	private void getStatistic(String place){
		ResultSet analyse;
		if(place != null){
			 analyse= iim.getDistanceAnalyse(place);
		}else{
			analyse = iim.getDistanceAnalyse();
		}
		try{
			analyse.first();
			String result = "<div style='font-family:sans-serif;font-size:12px'>";
			result += "Minimální vzdálenost sídla od lokality: <span style='color:" + getColor(analyse.getDouble("min")) + "'>" + analyse.getDouble("min") + "</span> km<br />";
			result += "Maximální vzdálenost sídla od lokality: <span style='color:" + getColor(analyse.getDouble("max")) + "'>" + analyse.getDouble("max") + "</span> km<br />";
			result += "</div>";
			www_analyze.getEngine().loadContent(utils.getHtmlHeader() + result + utils.getHtmlFoot());
			www_analyze.getEngine().reload();
		}catch(SQLException e){
			String descr = "Chyba při zobrazování statistiky";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	private String getColor(double distance){
		String color = "black";
		if(distance < 3){
			color = "green";
		}
		if(distance < 5 && distance >=3){
			color = "YELLOWGREEN";
		}
		if(distance < 10 && distance >=5){
			color = "Yellow ";
		}
		if(distance < 15 && distance >=10){
			color = "OrangeRed";
		}
		if(distance >= 15){
			color = "Red";
		}
		return color;
	}
	
	private void showResult(String result){
		www_analyze.getEngine().loadContent(utils.getHtmlHeader() + result +utils.getHtmlFoot());
		www_analyze.getEngine().reload();
	}
	
	/**
	 * Return first value from ArrayList with String[4] 
	 * @param name Searched string
	 * @param list ArrayList
	 * @return String
	 */
	private String[] getCoords(String name, ArrayList list){
		String[] coords = new String[2];
		for(int i = 0; i < list.size();i++){
			String[] values = (String[])list.get(i);
			if(name.equals(values[1])){
				coords[0] = values[2];
				coords[1] = values[3];
			}
		}
		return coords;
	}
	
	/*@Override
	protected void setTableColumns(){
		ResultSet rs = model.getData();
		table_data.getColumns().clear();
		List<String> columns = new ArrayList<>();
		try{
			ResultSetMetaData rsmd = rs.getMetaData();
			int ii = 0;
			while(ii< rsmd.getColumnCount()){
				ii++;
				columns.add(rsmd.getColumnLabel(ii));
			};
			int i = 0;
			for(String columnName : columns) {
				final int j = i;
				TableColumn col = new TableColumn(columnName);
				col.setCellValueFactory(new Callback<TableColumn.CellDataFeatures<ObservableList,String>,ObservableValue<String>>(){
					public ObservableValue<String> call(TableColumn.CellDataFeatures<ObservableList, String> param) {
						return new SimpleStringProperty(param.getValue().get(j).toString());
					}
				});
				if(columnName.equals("ID")){
					col.getStyleClass().add("columnid");
				}else if(columnName.equals("Maximální vzdálenost")){
					col.setCellValueFactory(cellData -> cellData.toString());
					col.setCellFactory(TextFieldTableCell.forTableColumn());
				}else{
					col.setCellFactory(TextFieldTableCell.forTableColumn());
				}
				columnsNames.put(columnName,i);
				table_data.getColumns().addAll(col);
				i++;
			}
		}catch(SQLException e){
			String descr = "Nezdařilo se nastavit sloupce pro tabulku";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}*/
}
