package org.hansv.vratickaimport3;

import com.google.inject.Singleton;
import java.net.URL;
import java.util.ArrayList;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.control.Tooltip;
import javafx.scene.input.KeyCode;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.PhytochorionModel;
import org.hansv.model.RestModel;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class PhytochorionController extends BaseController{

	@FXML
	private ProgressIndicator indicator_map_phytochorion;
	@FXML
	private Button reload_data;
	@FXML
	private Button map_phytochorions;
	@FXML
	private Button clear_phytochorions;
	@FXML
	private Button save_phytochorion;
	@FXML
	private Button delete_phytochorion;
	
	private PhytochorionModel phm;
	private RestModel rm;

	
	@Override
	public void initialize(URL location, ResourceBundle resources) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		utils = (Utils)context.getBean("utils");
		du = (DbUtils)context.getBean("dbUtils");
		rm = (RestModel)context.getBean("restModel");
		phm = (PhytochorionModel)context.getBean("phytochorionModel");
		model = (PhytochorionModel)context.getBean("phytochorionModel");
		if(du.tableExist("PHYTOCHORION") == false){
			reload_data.setDisable(true);
		}
		combo_value.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>(){
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if(combo_value.getEditor().getText().length() > 3 && t.getCode().isArrowKey() == false && t.getCode() != KeyCode.ENTER){
					comboThread(combo_value.getEditor().getText());
				}
			}
		});
		combo_value.getEditor().focusedProperty().addListener(
			(observable, oldValue, newValue) -> {
				if (newValue) {
					Platform.runLater(combo_value.getEditor()::selectAll);
				}
			}
		);
		Tooltip tp1 = new Tooltip();
		tp1.setText("Zobrazení dat");
		reload_data.setTooltip(tp1);
		Tooltip tp2 = new Tooltip();
		tp2.setText("Namapování fytochorionů");
		map_phytochorions.setTooltip(tp2);
		Tooltip tp3 = new Tooltip();
		tp3.setText("Zrušení namapování fytochorionů");
		clear_phytochorions.setTooltip(tp3);
		Tooltip tp4 = new Tooltip();
		tp4.setText("Přiřazení fytochorionu");
		save_phytochorion.setTooltip(tp4);
		Tooltip tp5 = new Tooltip();
		tp5.setText("Zrušení mapování fytochorionu");
		delete_phytochorion.setTooltip(tp5);
		handleReload();
		
	}
	
	@FXML
	public void handleReload(){
		if(du.tableExist("PHYTOCHORION")){
			setTableColumns();
			loadData();
		}
	}
	
	@FXML
	public void handleMap(){
		phm.setOnFailed((event)-> {
			Throwable t = phm.getException();
			String descr = "Chyba mapování fytochorionů";
			errlog.logErr(ExceptionUtils.getStackTrace(t), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(t);
			msg.showErr(errs[0] + errs[1], descr);
		});
		indicator_map_phytochorion.progressProperty().bind(phm.progressProperty());
		phm.reset();
		phm.setParams(data, rm);
		phm.start();
	}
	
	@FXML
	public void handleClear(){
		clearData();
	}
	
	@FXML
	public void handleSave(){
		save();
	}
	
	@FXML
	public void handleDelete(){
		delete();
	}
	
		
	private void comboThread(String sample){
		Thread t = new Thread(()->{
			Platform.runLater(new Runnable(){
				public void run(){
					combo(rm.getPhytochorion(sample));
				}
			});
		});
		t.start();
	}
}
