package org.hansv.vratickaimport3;


import com.google.inject.Singleton;
import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.control.Tooltip;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.ProjectModel;
import org.hansv.model.RestModel;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;


/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class ProjectController extends BaseController{

	@FXML
	private ProgressIndicator indicator_map_project;
	@FXML
	private Button reload_data;
	@FXML
	private Button map_projects;
	@FXML
	private Button clear_projects;
	@FXML
	private Button save_project;
	@FXML
	private Button delete_project;
	@FXML
	private Button add_project;
	

	private ProjectModel pm;
	private RestModel rm;
	
	
	@Override
	public void initialize(URL url, ResourceBundle rb) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		pm = (ProjectModel)context.getBean("projectModel");
		model = (ProjectModel)context.getBean("projectModel");
		utils = (Utils)context.getBean("utils");
		du = (DbUtils)context.getBean("dbUtils");
		rm = (RestModel)context.getBean("restModel");
		if(du.tableExist("PROJECT") == false){
			reload_data.setDisable(true);
		}
		combo_value.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>(){
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if(combo_value.getEditor().getText().length() > 2 && t.getCode().isArrowKey() == false && t.getCode() != KeyCode.ENTER){
					comboThread(combo_value.getEditor().getText());
				}
			}
		});
		combo_value.getEditor().focusedProperty().addListener(
			(observable, oldValue, newValue) -> {
				if (newValue) {
					Platform.runLater(combo_value.getEditor()::selectAll);
				}
			}
		);
		Tooltip tp1 = new Tooltip();
		tp1.setText("Zobrazení dat");
		reload_data.setTooltip(tp1);
		Tooltip tp2 = new Tooltip();
		tp2.setText("Namapování projektů");
		map_projects.setTooltip(tp2);
		Tooltip tp3 = new Tooltip();
		tp3.setText("Zrušení přiřazení projektů");
		clear_projects.setTooltip(tp3);
		Tooltip tp4 = new Tooltip();
		tp4.setText("Přiřazení projektu");
		save_project.setTooltip(tp4);
		Tooltip tp5 = new Tooltip();
		tp5.setText("Zrušení přiřazení projektu");
		delete_project.setTooltip(tp5);
		Tooltip tp6 = new Tooltip();
		tp6.setText("Nový projekt");
		add_project.setTooltip(tp6);
		handleReload();
	}
	
	@FXML
	public void handleReload(){
		if(du.tableExist("PROJECT")){
			setTableColumns();
			loadData();
		}
	}
	
	@FXML
	public void handleClear(){
		clearData();
	}
	
	@FXML
	public void handleMap(){
		pm.setOnFailed((event)-> {
			Throwable t = pm.getException();
			String descr = "Chyba mapování projektů";
			errlog.logErr(ExceptionUtils.getStackTrace(t), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(t);
			msg.showErr(errs[0] + errs[1], descr);
		});
		indicator_map_project.progressProperty().bind(pm.progressProperty());
		pm.reset();
		pm.setParams(data, rm);
		pm.start();
	}
	
	@FXML
	public void handleSave(){
		save();
	}
	
	@FXML
	public void handleDelete(){
		delete();
	}
	
	@FXML
	public void handleAdd(ActionEvent event){
		try {
			FXMLLoader loader = new FXMLLoader(getClass().getResource("/fxml/WWWBrowser.fxml"));
			Stage stage = new Stage(StageStyle.DECORATED);
			stage.setScene(new Scene((Pane) loader.load()));
			WWWBrowserController controller = loader.<WWWBrowserController>getController();
			controller.load("project/editremote");
			stage.show();
			
		} catch (IOException e) {
			String descr = "Nezdařilo se otevřít okno s vestavěným WWW prohlížečem";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		
	}
	
	
	
	private void comboThread(String sample){
		Thread t = new Thread(()->{
			Platform.runLater(new Runnable(){
				public void run(){
					combo(rm.getProjects(sample));
				}
			});
		});
		t.start();
	}
	
}
