
package org.hansv.vratickaimport3;

import com.google.inject.Guice;
import com.google.inject.Injector;
import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.fxml.JavaFXBuilderFactory;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.PasswordField;
import javafx.scene.control.TextField;
import javafx.scene.control.Tooltip;
import javafx.stage.Stage;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.ProxyModel;
import org.hansv.model.RestModel;
import org.hansv.utils.DepInjectionModule;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.FXMLControllersLoader;
import org.hansv.utils.LoginSingleton;
import org.hansv.utils.Messages;
import org.hansv.utils.ProxySingleton;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class ProxysettingController implements Initializable {

	@FXML
	private TextField host;
	@FXML
	private TextField port;
	@FXML
	private TextField user;
	@FXML
	private PasswordField password;
	@FXML
	private Button savesettings;
	@FXML
	private Button testnet;
	
	private RestModel rm;
	private Utils utils;
	private Messages msg;
	private ErrorLog errlog;
	private ProxySingleton ps;
	private ProxyModel pm;
	/**
	 * Initializes the controller class.
	 */
	@Override
	public void initialize(URL url, ResourceBundle rb) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		rm = (RestModel)context.getBean("restModel");
		utils = (Utils)context.getBean("utils");
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		ps = ProxySingleton.getInstance();
		pm = (ProxyModel)context.getBean("proxyModel");
		Tooltip tp1 = new Tooltip();
		tp1.setText("Uložit konfiguraci");
		savesettings.setTooltip(tp1);
		Tooltip tp2 = new Tooltip();
		tp2.setText("Test spojení");
		testnet.setTooltip(tp2);
		host.setText(ps.getHost());
		if(ps.getPort() > 0){
			port.setText(String.valueOf(ps.getPort()));
		}else{
			port.setText("");
		}
		if(ps.getUser() != null){
			if(ps.getUser() .length() > 0){
				user.setText(ps.getUser());
			}
		}
		if(ps.getPassword() != null){
			if(ps.getPassword().length() >0){
				password.setText(ps.getPassword());
			}
		}
	}	
	
	@FXML
	public void handleTest(){
		try{
			setProxySingleton();
			String test = rm.getTheme();
			if(test != null){
				msg.showInfo("Test spojení", "Test spojení proběhl v pořádku");
			}else{
				msg.showWarn("Test spojení", "Nelze se spojit se serverem");
			}
		}catch(IOException e){
			String descr = "IOException, ProxysettingsController.handleTest";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	@FXML
	public void handleSave(){
		if(port.getText().length() >0){
			if(utils.isInteger(port.getText()) == false){
				msg.showWarn("", "Zadaný port není celé číslo");
				return;
			}
		}
		if(user.getText() != null && password.getText() == null){
			msg.showWarn("Prázdné heslo", "Zadejte heslo");
			return;
		}
		if(user.getText() == null && password.getText() != null){
			msg.showWarn("", "Zadejte jméno");
			return;
		}
		if((user.getText() != null || password.getText() != null) && host.getText()== null){
			if(user.getText().length() > 0 || password.getText().length() > 0){;
				msg.showWarn("", "Zadejte proxy server");
				return;
			}
		}
		if(host.getText() != null && port.getText() == null){
			msg.showWarn("", "Zadejte port");
			return;
		}
		if(host.getText() == null && port.getText() != null){
			if(port.getText().length() > 0){
				msg.showWarn("", "Zadejte proxy server");
				return;
			}
		}
		setProxySingleton();
		pm.setConfig(host.getText(), port.getText(), user.getText(), password.getText());
		Stage stageLogin = (Stage) savesettings.getScene().getWindow();
		stageLogin.close();
		showLogin();
	}
	
	private void setProxySingleton(){
		if(host.getText() != null){
			if(host.getText().length() > 0){
				ps.setHost(host.getText());
			}else{
				ps.setHost(null);
			}
		}else{
			ps.setHost(null);
		}
		if(port.getText() != null){
			if(port.getText().length() >0){
				if(utils.isInteger(port.getText())){
					ps.setPort(Integer.parseInt(port.getText()));
				}else{
					msg.showWarn("Chybný formát čísla", "Číslo portu není celé číslo");
					return;
				}
			}else{
				ps.setPort(-1);
			}
		}else{
			ps.setPort( -1 );
		}
		if(user.getText() != null){
			if(user.getText().length() >0){
				ps.setUser(user.getText());
			}else{
				ps.setUser(null);
			}
		}else{
			ps.setUser(null);
		}
		if(password.getText() != null){
			if(password.getText() != null){
				if(password.getText().length() > 0){
					ps.setPassword(password.getText());
				}else{
					ps.setPassword(null);
				}
			}else{
				ps.setPassword(null);
			}
		}else{
			ps.setPassword(null);
		}
	}
	
	private void showLogin(){
		try {
			final Injector injector = Guice.createInjector(new DepInjectionModule());
			final FXMLLoader loader = new FXMLControllersLoader(getClass().getResource("/fxml/Login.fxml"), null, new JavaFXBuilderFactory(), injector);
			final Parent view = loader.load();
			Scene scene = new Scene(view);
			Stage stageData = new Stage();
			String theme = rm.getTheme();
			String url = utils.getServerPath() + "css/"+ theme +".css";
			scene.getStylesheets().add("/styles/Styles.css");
			scene.getStylesheets().add(url);
			stageData.setTitle("Vratička import");
			stageData.setScene(scene);
			LoginSingleton ls = LoginSingleton.getInstance();
			stageData.show();
		} catch (IOException e) {
			String descr = "IOException, ProxysettingsController.showLogin";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
}
