package org.hansv.vratickaimport3;

import com.google.inject.Singleton;
import java.net.URL;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Tooltip;
import javafx.scene.input.KeyCode;
import org.hansv.model.RestModel;
import org.hansv.model.SourceModel;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;


/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class SourceController extends BaseController{

	@FXML
	private Button reload_data;
	@FXML
	private Button clear_sources;
	@FXML
	private Button save_source;
	@FXML
	private Button delete_source;
	@FXML
	private Button add_source;
	private RestModel rm;
	
	@Override
	public void initialize(URL location, ResourceBundle resources) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		model = (SourceModel)context.getBean("sourceModel");
		utils = (Utils)context.getBean("utils");
		du = (DbUtils)context.getBean("dbUtils");
		rm = (RestModel)context.getBean("restModel");
		if(du.tableExist("SOURCE") == false){
			reload_data.setDisable(true);
		}
		combo_value.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>(){
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if(combo_value.getEditor().getText().length() > 2 && t.getCode().isArrowKey() == false && t.getCode() != KeyCode.ENTER){
					comboThread(combo_value.getEditor().getText());
				}
			}
			
		});
		combo_value.getEditor().focusedProperty().addListener(
			(observable, oldValue, newValue) -> {
				if (newValue) {
					Platform.runLater(combo_value.getEditor()::selectAll);
				}
			}
		);
		Tooltip tp1 = new Tooltip();
		tp1.setText("Zobrazení dat");
		reload_data.setTooltip(tp1);
		Tooltip tp2 = new Tooltip();
		tp2.setText("Zrušení přiřazení zddrojů");
		clear_sources.setTooltip(tp2);
		Tooltip tp3 = new Tooltip();
		tp3.setText("Přiřazení zdroje");
		save_source.setTooltip(tp3);
		Tooltip tp4 = new Tooltip();
		tp4.setText("Zrušení přiřazení zdroje");
		delete_source.setTooltip(tp4);
		Tooltip tp5 = new Tooltip();
		tp5.setText("Nový zdroj");
		add_source.setTooltip(tp5);
		handleReload();
	}
	
	@FXML
	public void handleReload(){
		if(du.tableExist("SOURCE")){
			setTableColumns();
			loadData();
		}
	}
	
	@FXML
	public void handleClear(){
		clearData();
	}
	
	@FXML
	public void handleSave(){
		save();
	}
	
	@FXML
	public void handleDelete(){
		delete();
	}
	
	@FXML
	public void handleAdd(ActionEvent event){
		add("source/editremote");
	}
	
	private void comboThread(String sample){
		Thread t = new Thread(()->{
			Platform.runLater(new Runnable(){
				public void run(){
					combo(rm.getSources(sample));
				}
			});
		});
		t.start();
	}
}

