package org.hansv.vratickaimport3;

import com.google.inject.Singleton;
import java.net.URL;
import java.sql.ResultSet;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.concurrent.Task;
import javafx.concurrent.WorkerStateEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.Cursor;
import javafx.scene.control.Button;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.control.Tooltip;
import javafx.scene.input.KeyCode;
import javafx.stage.Stage;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.InputImportModel;
import org.hansv.model.RestModel;
import org.hansv.model.TaxonModel;
import org.hansv.utils.DbUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.hansv.utils.Utils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
@Singleton
public class TaxonController extends BaseController {

	@FXML
	public Button reload_taxon_data;
	@FXML
	private Button save_taxon;
	@FXML
	private Button delete_taxon;
	@FXML
	private Button map_taxons;
	@FXML
	private Button clear_taxons;
	@FXML
	private ProgressIndicator indicator_map_taxons;
	@FXML
	private Button repair_taxons;

	private TaxonModel tm;
	private RestModel rm;
	private InputImportModel iim;

	/**
	 * Initializes the controller class.
	 */
	@Override
	public void initialize(URL url, ResourceBundle rb) {
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		tm = (TaxonModel)context.getBean("taxonModel");
		rm = (RestModel)context.getBean("restModel");
		iim = (InputImportModel)context.getBean("inputImportModel");
		model = (TaxonModel)context.getBean("taxonModel");
		du = (DbUtils)context.getBean("dbUtils");
		utils = (Utils)context.getBean("utils");
		utils.setUp();
		if(du.tableExist("TAXON") == false){
			reload_taxon_data.setDisable(true);
		}
		Tooltip tp1 = new Tooltip();
		tp1.setText("Zobrazení dat");
		reload_taxon_data.setTooltip(tp1);
		Tooltip tp2 = new Tooltip();
		tp2.setText("Přiřazení taxonu");
		save_taxon.setTooltip(tp2);
		Tooltip tp3 = new Tooltip();
		tp3.setText("Zrušení přiřazení taxonu");
		delete_taxon.setTooltip(tp3);
		Tooltip tp4 = new Tooltip();
		tp4.setText("Namapování taxonů");
		map_taxons.setTooltip(tp4);
		Tooltip tp5 = new Tooltip();
		tp5.setText("Zrušení namapování taxonů");
		clear_taxons.setTooltip(tp5);
		Tooltip tp6 = new Tooltip();
		tp6.setText("Oprava názvů taxonů");
		repair_taxons.setTooltip(tp6);
		combo_value.setOnKeyReleased(new EventHandler<javafx.scene.input.KeyEvent>(){
			@Override
			public void handle(javafx.scene.input.KeyEvent t) {
				if(combo_value.getEditor().getText().length() > 4 && t.getCode().isArrowKey() == false && t.getCode() != KeyCode.ENTER){
					comboThread(combo_value.getEditor().getText());
				}
			}
			
		});
		combo_value.getEditor().focusedProperty().addListener(
			(observable, oldValue, newValue) -> {
				if (newValue) {
					Platform.runLater(combo_value.getEditor()::selectAll);
				}
			}
		);
		handleReload();
	}

	public void handleReload(){
		if(du.tableExist("TAXON") == true){
			setTableColumns();
			loadData();
		}
	}

	
	@FXML
	public void handleSave(){
		save();
	}
	
	@FXML
	public void handleDelete(){
		delete();
	}
	
	private void comboThread(String sample){
		Thread t = new Thread(()->{
			Platform.runLater(new Runnable(){
				public void run(){
					combo(rm.getTaxons(sample));
				}
			});
		});
		t.start();
	}
	
	@FXML
	public void handleMap(){
		tm.setOnFailed((event)-> {
			Throwable t = tm.getException();
			String descr = "Chyba mapování taxonů";
			errlog.logErr(ExceptionUtils.getStackTrace(t), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(t);
			msg.showErr(descr,errs[0] + errs[1]);
		});
		indicator_map_taxons.progressProperty().bind(tm.progressProperty());
		tm.reset();
		tm.setParams(data, rm);
		tm.start();
	}

	@FXML
	public void handleClear(){
		clearData();
	}
	
	@FXML
	public void handleRepair(){
		Task task = new Task() {
			@Override
			protected Object call() throws Exception {
				repair_taxons.setDisable(true);
				Stage stage = (Stage)repair_taxons.getScene().getWindow();
				stage.getScene().setCursor(Cursor.WAIT);
				ResultSet rs = iim.getTaxon();
				rs.last();
				int count = rs.getRow();
				rs.beforeFirst();
				String taxon;
				String[] taxon2;
				int i = 0;
				while(rs.next()){
					taxon = rs.getString("taxon");
					taxon = utils.delMoreSpace(taxon);
					taxon = utils.rewriteHybridMarker(taxon);
					iim.saveTaxon(taxon, rs.getInt("id_input"),null);
					taxon2 = utils.prepareCf(taxon);
					if(taxon2[0] != null){
						iim.saveTaxon(taxon2[0], rs.getInt("id_input"),taxon2[1]);
					}
					updateProgress(count, i);
					i++;
				}
				tm.createTable();
				tm.importData();
				stage.getScene().setCursor(Cursor.DEFAULT);
				repair_taxons.setDisable(false);
				return "Success";
			}
		};
		new Thread(task).start();
		indicator_map_taxons.progressProperty().bind(task.progressProperty());
		task.setOnSucceeded(new EventHandler<WorkerStateEvent>() {
			@Override
			public void handle(WorkerStateEvent event) {
				handleReload();
			}
		});
	}
}
