/*
 * @license © 2015, Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 */

package org.hansv.utils;

import java.util.ArrayList;
import java.util.List;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author merkur
 */
public class UtilsTest {
	
	public UtilsTest() {
	}
	
	@BeforeClass
	public static void setUpClass() {
	}
	
	@AfterClass
	public static void tearDownClass() {
	}
	
	@Before
	public void setUp() {
	}
	
	@Test
	public void isDuplicitInListTest1(){
		Utils instance = new Utils();
		List<String> al = new ArrayList<>();
		al.add("Taxon");
		al.add("Lokalita");
		al.add("longitude");
		al.add("latitude");
		al.add("Taxon");
		al.add("altitude");
		List result = instance.getDuplicitInList(al);
		assertEquals(false,result.isEmpty());
		assertEquals(1, result.size());
	}
	
	@Test
	public void isDuplicitInListTest2(){
		Utils instance = new Utils();
		List<String> al = new ArrayList<>();
		al.add("Taxon");
		al.add("Lokalita");
		al.add("longitude");
		al.add("latitude");
		al.add("Nalezce");
		al.add("altitude");
		List result = instance.getDuplicitInList(al);
		assertEquals(true,result.isEmpty());
	}
	
	@Test
	public void transformLongitude_1(){
		Utils u = new Utils();
		String coord = "S14°32.8'";
		assertEquals(null, -14.546666666666667,u.transformLongitude(coord), 0);
	}
	
	@Test
	public void transformLongitude_2(){
		Utils u = new Utils();
		String coord = "14°25′32.8″S";
		assertEquals(null, -14.425777777777776,u.transformLongitude(coord), 0);
	}
	
	@Test
	public void transformLongitude_3(){
		Utils u = new Utils();
		String coord = "-14.25624S";
		assertEquals(null, 14.25624,u.transformLongitude(coord), 0);
	}
	
	@Test
	public void transformLongitude_4(){
		Utils u = new Utils();
		String coord = "14:25:32.8N";
		assertEquals(null, 14.425777777777776,u.transformLongitude(coord), 0);
	}
	
	@Test
	public void transformLongitude_5(){
		Utils u = new Utils();
		String coord = "14:25:32N";
		assertEquals(null, 14.425555555555555,u.transformLongitude(coord), 0);
	}
	
	@Test
	public void transformLongitude_6(){
		Utils u = new Utils();
		String coord = "14:25:32";
		assertEquals(null, 14.425555555555555,u.transformLongitude(coord), 0);
	}
	
	@Test
	public void transformLongitude_7(){
		Utils u = new Utils();
		String coord = "14:25:32.8";
		assertEquals(null, 14.425777777777776,u.transformLongitude(coord), 0);
	}
	
	@Test
	public void transformLongitude_8(){
		Utils u = new Utils();
		String coord = "48°54’17,64\"";
		assertEquals(null, 48.9049,u.transformLongitude(coord), 0);
	}
	
	@Test
	public void transformLongitude_9(){
		Utils u = new Utils();
		String coord = "13°59'42\"";
		assertEquals(null, 13.995,u.transformLongitude(coord), 0);
	}
	
	@Test
	public void transformLongitude_10(){
		Utils u = new Utils();
		String coord = "190°59'42\"";
		assertNull(u.transformLongitude(coord));
	}
	
	@Test
	public void transformLatitude(){
		Utils u = new Utils();
		String coord = "90°59'42\"";
		assertNull(u.transformLatitude(coord));
	}
	
	@Test
	public void transformDate_1(){
		Utils u = new Utils();
		String coord = "2000-12-25";
		assertEquals(null, "2000-12-25",u.transformDate(coord));
	}
	
	@Test
	public void transformDate_2(){
		Utils u = new Utils();
		String coord = "25. 12. 2015";
		assertEquals(null, "2015-12-25",u.transformDate(coord));
	}
	
	@Test
	public void transformDate_3(){
		Utils u = new Utils();
		String coord = "2015";
		assertEquals(null, "2015-00-00",u.transformDate(coord));
	}
	
	@Test
	public void transformDate_4(){
		Utils u = new Utils();
		String coord = "2015-05";
		assertEquals(null, "2015-05-00",u.transformDate(coord));
	}
	
	@Test
	public void transformDate_5(){
		Utils u = new Utils();
		String coord = "s.d.";
		assertEquals(null, "1000-01-01",u.transformDate(coord));
	}
	
	@Test
	public void getDistance(){
		Utils u = new Utils();
		double distance = u.getDistance(48.983651, 14.473923, 48.986488, 14.475007);
		assertEquals(0.325, distance, 0);
	}
	
	@Test
	public void rewriteHybridMarker1(){
		Utils u = new Utils();
		String taxon = u.rewriteHybridMarker("Carex x allolepis");
		assertEquals("Carex × allolepis", taxon);
	}
	
	@Test
	public void rewriteHybridMarker2(){
		Utils u = new Utils();
		String taxon = u.rewriteHybridMarker("Carex ×allolepis");
		assertEquals("Carex × allolepis", taxon);
	}
	
	@Test
	public void rewriteHybridMarker3(){
		Utils u = new Utils();
		String taxon = u.rewriteHybridMarker("Carex allolepis");
		assertEquals("Carex allolepis", taxon);
	}
	
	@Test
	public void delMoreSpace(){
		Utils u = new Utils();
		String taxon = u.delMoreSpace("Carex   allolepis ");
		assertEquals("Carex allolepis", taxon);
	}
	
	@Test
	public void prepareCf(){
		Utils u = new Utils();
		String[] taxon = u.prepareCf("Carex  cf. allolepis ");
		assertEquals("Carex allolepis", taxon[0]);
		assertEquals("cf.", taxon[1]);
	}
	
	@After
	public void tearDown() {
	}
}
